switch (u8(m_ref >> 16)) // prefix
{
case 0xff:
{
	switch (u8(m_ref >> 8)) // opcode
	{
	case 0xff:
		switch (u8(m_ref))
		{
		case 0x00:
			if (m_busrq_state) {
				if (!m_busack_state) {
					m_busack_state = 1;
					m_busack_cb(1);
				}
				if (m_icount > 0)
					m_icount = 0;
				return;
			} else if (m_busack_state) {
				m_busack_state = 0;
				m_busack_cb(0);
			}
			if (m_nmi_pending) {
				if (m_stackless) {
					// on take_nmi
					m_out_nextreg_cb(0xc2, m_pc.b.l);
					m_out_nextreg_cb(0xc3, m_pc.b.h);
				}
			// Check if processor was halted
			leave_halt();
			#if HAS_LDAIR_QUIRK
				// reset parity flag after LD A,I or LD A,R
				if (m_after_ldair) F &= ~PF;
			#endif
			m_iff1 = 0;
			m_r++;
			m_icount -= 5;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			PC = 0x0066;
			WZ = PC;
			m_nmi_pending = false;
			} else if (m_irq_state != CLEAR_LINE && m_iff1 && !m_after_ei) {
			// check if processor was halted
			leave_halt();
			// clear both interrupt flip flops
			m_iff1 = m_iff2 = 0;
			// say hi
			// Not precise in all cases. z80 must finish current instruction (NOP) to reach this state - in such case frame timings are shifter from cb event if calculated based on it.
			m_irqack_cb(1);
			m_r++;
			{
				// fetch the IRQ vector
				m_irqfetch_cb(1);
				device_z80daisy_interface *intf = daisy_get_irq_device();
				m_tmp_irq_vector = (intf != nullptr) ? intf->z80daisy_irq_ack() : standard_irq_callback(0, m_pc.w);
				LOGMASKED(LOG_INT, "single INT m_tmp_irq_vector $%02x\n", m_tmp_irq_vector);
			}
			// 'interrupt latency' cycles
			m_icount -= 2;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			// Interrupt mode 2. Call [i:databyte]
			if (m_im == 2) {
				// Zilog's datasheet claims that "the least-significant bit must be a zero."
				// However, experiments have confirmed that IM 2 vectors do not have to be
				// even, and all 8 bits will be used; even $FF is handled normally.
				// CALL opcode timing
				m_icount -= 5;
				if (m_icount <= 0) {
					m_ref = (m_ref & 0xffff00) | 0x07;
					return;
				}
				[[fallthrough]];
		case 0x07:
				TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			SP--;
			[[fallthrough]];
		case 0x0a:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0a;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
				m_tmp_irq_vector = (m_tmp_irq_vector & 0xff) | (m_i << 8);
				TADR = m_tmp_irq_vector;
			[[fallthrough]];
		case 0x0c:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0d;
				return;
			}
			[[fallthrough]];
		case 0x0d:
			TDAT_H = TDAT_L; TADR++;
			[[fallthrough]];
		case 0x0e:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0e;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0f;
				return;
			}
			[[fallthrough]];
		case 0x0f:
			using std::swap; std::swap(TDAT_H, TDAT_L);
				PC = TDAT;
				LOGMASKED(LOG_INT, "IM2 [$%04x] = $%04x\n", m_tmp_irq_vector, PC);
			} else if (m_im == 1) {
				// Interrupt mode 1. RST 38h
				LOGMASKED(LOG_INT, "'%s' IM1 $0038\n", tag());
				// RST $38
				m_icount -= 5;
				if (m_icount <= 0) {
					m_ref = (m_ref & 0xffff00) | 0x10;
					return;
				}
				[[fallthrough]];
		case 0x10:
				TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x11:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x11;
				} else
					m_ref = (m_ref & 0xffff00) | 0x12;
				return;
			}
			[[fallthrough]];
		case 0x12:
			SP--;
			[[fallthrough]];
		case 0x13:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x13;
				} else
					m_ref = (m_ref & 0xffff00) | 0x14;
				return;
			}
			[[fallthrough]];
		case 0x14:
				PC = 0x0038;
			} else {
				/* Interrupt mode 0. We check for CALL and JP instructions,
				   if neither of these were found we assume a 1 byte opcode
				   was placed on the databus */
				LOGMASKED(LOG_INT, "IM0 $%04x\n", m_tmp_irq_vector);
				// check for nop
				if (m_tmp_irq_vector != 0x00) {
					if ((m_tmp_irq_vector & 0xff0000) == 0xcd0000) {
						// CALL $xxxx cycles
						m_icount -= 11;
						if (m_icount <= 0) {
							m_ref = (m_ref & 0xffff00) | 0x15;
							return;
						}
						[[fallthrough]];
		case 0x15:
						TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x16:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x16;
				} else
					m_ref = (m_ref & 0xffff00) | 0x17;
				return;
			}
			[[fallthrough]];
		case 0x17:
			SP--;
			[[fallthrough]];
		case 0x18:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x18;
				} else
					m_ref = (m_ref & 0xffff00) | 0x19;
				return;
			}
			[[fallthrough]];
		case 0x19:
						PC = m_tmp_irq_vector & 0xffff;
					} else if ((m_tmp_irq_vector & 0xff0000) == 0xc30000) {
						// JP $xxxx cycles
						m_icount -= 10;
						if (m_icount <= 0) {
							m_ref = (m_ref & 0xffff00) | 0x1a;
							return;
						}
						[[fallthrough]];
		case 0x1a:
						PC = m_tmp_irq_vector & 0xffff;
					} else if ((m_tmp_irq_vector & 0xc7) == 0xc7) {
						// RST $xx cycles
						m_icount -= 5;
						if (m_icount <= 0) {
							m_ref = (m_ref & 0xffff00) | 0x1b;
							return;
						}
						[[fallthrough]];
		case 0x1b:
						TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x1c:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x1c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x1d;
				return;
			}
			[[fallthrough]];
		case 0x1d:
			SP--;
			[[fallthrough]];
		case 0x1e:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x1e;
				} else
					m_ref = (m_ref & 0xffff00) | 0x1f;
				return;
			}
			[[fallthrough]];
		case 0x1f:
						PC = m_tmp_irq_vector & 0x0038;
					} else if (m_tmp_irq_vector == 0xfb) {
						// EI cycles
						m_icount -= 4;
						if (m_icount <= 0) {
							m_ref = (m_ref & 0xffff00) | 0x20;
							return;
						}
						[[fallthrough]];
		case 0x20:
						ei();
					} else {
						logerror("take_interrupt: unexpected opcode in im0 mode: 0x%02x\n", m_tmp_irq_vector);
					}
				}
			}
			WZ = PC;
			#if HAS_LDAIR_QUIRK
				// reset parity flag after LD A,I or LD A,R
				if (m_after_ldair) F &= ~PF;
			#endif
			}
			m_after_ei = false;
			m_after_ldair = false;
			if (m_halt) {
				debugger_wait_hook();
			[[fallthrough]];
		case 0x21:
			TDAT8 = opcode_read();
			m_icount -= m_m1_cycles-2;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_m1_cycles-2;
					m_ref = (m_ref & 0xffff00) | 0x21;
				} else
					m_ref = (m_ref & 0xffff00) | 0x22;
				return;
			}
			[[fallthrough]];
		case 0x22:
			m_refresh_cb((I << 8) | (R2 & 0x80) | (R & 0x7f), 0x00, 0xff);
			m_icount -= 2;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x23;
				return;
			}
			[[fallthrough]];
		case 0x23:
			PC++; R++; Q = QT; QT = YF | XF;
				PC--;
				m_ref = 0xffff00;
				return;
			} else {
				PRVPC = PC;
				debugger_instruction_hook(PC);
			[[fallthrough]];
		case 0x24:
			TDAT8 = opcode_read();
			m_icount -= m_m1_cycles-2;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_m1_cycles-2;
					m_ref = (m_ref & 0xffff00) | 0x24;
				} else
					m_ref = (m_ref & 0xffff00) | 0x25;
				return;
			}
			[[fallthrough]];
		case 0x25:
			m_refresh_cb((I << 8) | (R2 & 0x80) | (R & 0x7f), 0x00, 0xff);
			m_icount -= 2;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x26;
				return;
			}
			[[fallthrough]];
		case 0x26:
			PC++; R++; Q = QT; QT = YF | XF;
				m_ref = (0x00 << 16) | (TDAT8 << 8);
				return;
			}
			break;

		}
		break;


	}
		break;

}
break; // prefix: 0xff

case 0xcb:
{
	switch (u8(m_ref >> 8)) // opcode
	{
	case 0x00:
			B = rlc(B);
		break;

	case 0x01:
			C = rlc(C);
		break;

	case 0x02:
			D = rlc(D);
		break;

	case 0x03:
			E = rlc(E);
		break;

	case 0x04:
			H = rlc(H);
		break;

	case 0x05:
			L = rlc(L);
		break;

	case 0x06:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = rlc(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x07:
			A = rlc(A);
		break;

	case 0x08:
			B = rrc(B);
		break;

	case 0x09:
			C = rrc(C);
		break;

	case 0x0a:
			D = rrc(D);
		break;

	case 0x0b:
			E = rrc(E);
		break;

	case 0x0c:
			H = rrc(H);
		break;

	case 0x0d:
			L = rrc(L);
		break;

	case 0x0e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = rrc(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x0f:
			A = rrc(A);
		break;

	case 0x10:
			B = rl(B);
		break;

	case 0x11:
			C = rl(C);
		break;

	case 0x12:
			D = rl(D);
		break;

	case 0x13:
			E = rl(E);
		break;

	case 0x14:
			H = rl(H);
		break;

	case 0x15:
			L = rl(L);
		break;

	case 0x16:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = rl(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x17:
			A = rl(A);
		break;

	case 0x18:
			B = rr(B);
		break;

	case 0x19:
			C = rr(C);
		break;

	case 0x1a:
			D = rr(D);
		break;

	case 0x1b:
			E = rr(E);
		break;

	case 0x1c:
			H = rr(H);
		break;

	case 0x1d:
			L = rr(L);
		break;

	case 0x1e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = rr(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x1f:
			A = rr(A);
		break;

	case 0x20:
			B = sla(B);
		break;

	case 0x21:
			C = sla(C);
		break;

	case 0x22:
			D = sla(D);
		break;

	case 0x23:
			E = sla(E);
		break;

	case 0x24:
			H = sla(H);
		break;

	case 0x25:
			L = sla(L);
		break;

	case 0x26:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = sla(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x27:
			A = sla(A);
		break;

	case 0x28:
			B = sra(B);
		break;

	case 0x29:
			C = sra(C);
		break;

	case 0x2a:
			D = sra(D);
		break;

	case 0x2b:
			E = sra(E);
		break;

	case 0x2c:
			H = sra(H);
		break;

	case 0x2d:
			L = sra(L);
		break;

	case 0x2e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = sra(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x2f:
			A = sra(A);
		break;

	case 0x30:
			B = sll(B);
		break;

	case 0x31:
			C = sll(C);
		break;

	case 0x32:
			D = sll(D);
		break;

	case 0x33:
			E = sll(E);
		break;

	case 0x34:
			H = sll(H);
		break;

	case 0x35:
			L = sll(L);
		break;

	case 0x36:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = sll(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x37:
			A = sll(A);
		break;

	case 0x38:
			B = srl(B);
		break;

	case 0x39:
			C = srl(C);
		break;

	case 0x3a:
			D = srl(D);
		break;

	case 0x3b:
			E = srl(E);
		break;

	case 0x3c:
			H = srl(H);
		break;

	case 0x3d:
			L = srl(L);
		break;

	case 0x3e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = srl(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x3f:
			A = srl(A);
		break;

	case 0x40:
			bit(0, B);
		break;

	case 0x41:
			bit(0, C);
		break;

	case 0x42:
			bit(0, D);
		break;

	case 0x43:
			bit(0, E);
		break;

	case 0x44:
			bit(0, H);
		break;

	case 0x45:
			bit(0, L);
		break;

	case 0x46:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_hl(0, TDAT8);
			break;

		}
		break;

	case 0x47:
			bit(0, A);
		break;

	case 0x48:
			bit(1, B);
		break;

	case 0x49:
			bit(1, C);
		break;

	case 0x4a:
			bit(1, D);
		break;

	case 0x4b:
			bit(1, E);
		break;

	case 0x4c:
			bit(1, H);
		break;

	case 0x4d:
			bit(1, L);
		break;

	case 0x4e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_hl(1, TDAT8);
			break;

		}
		break;

	case 0x4f:
			bit(1, A);
		break;

	case 0x50:
			bit(2, B);
		break;

	case 0x51:
			bit(2, C);
		break;

	case 0x52:
			bit(2, D);
		break;

	case 0x53:
			bit(2, E);
		break;

	case 0x54:
			bit(2, H);
		break;

	case 0x55:
			bit(2, L);
		break;

	case 0x56:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_hl(2, TDAT8);
			break;

		}
		break;

	case 0x57:
			bit(2, A);
		break;

	case 0x58:
			bit(3, B);
		break;

	case 0x59:
			bit(3, C);
		break;

	case 0x5a:
			bit(3, D);
		break;

	case 0x5b:
			bit(3, E);
		break;

	case 0x5c:
			bit(3, H);
		break;

	case 0x5d:
			bit(3, L);
		break;

	case 0x5e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_hl(3, TDAT8);
			break;

		}
		break;

	case 0x5f:
			bit(3, A);
		break;

	case 0x60:
			bit(4, B);
		break;

	case 0x61:
			bit(4, C);
		break;

	case 0x62:
			bit(4, D);
		break;

	case 0x63:
			bit(4, E);
		break;

	case 0x64:
			bit(4, H);
		break;

	case 0x65:
			bit(4, L);
		break;

	case 0x66:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_hl(4, TDAT8);
			break;

		}
		break;

	case 0x67:
			bit(4, A);
		break;

	case 0x68:
			bit(5, B);
		break;

	case 0x69:
			bit(5, C);
		break;

	case 0x6a:
			bit(5, D);
		break;

	case 0x6b:
			bit(5, E);
		break;

	case 0x6c:
			bit(5, H);
		break;

	case 0x6d:
			bit(5, L);
		break;

	case 0x6e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_hl(5, TDAT8);
			break;

		}
		break;

	case 0x6f:
			bit(5, A);
		break;

	case 0x70:
			bit(6, B);
		break;

	case 0x71:
			bit(6, C);
		break;

	case 0x72:
			bit(6, D);
		break;

	case 0x73:
			bit(6, E);
		break;

	case 0x74:
			bit(6, H);
		break;

	case 0x75:
			bit(6, L);
		break;

	case 0x76:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_hl(6, TDAT8);
			break;

		}
		break;

	case 0x77:
			bit(6, A);
		break;

	case 0x78:
			bit(7, B);
		break;

	case 0x79:
			bit(7, C);
		break;

	case 0x7a:
			bit(7, D);
		break;

	case 0x7b:
			bit(7, E);
		break;

	case 0x7c:
			bit(7, H);
		break;

	case 0x7d:
			bit(7, L);
		break;

	case 0x7e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_hl(7, TDAT8);
			break;

		}
		break;

	case 0x7f:
			bit(7, A);
		break;

	case 0x80:
			B = res(0, B);
		break;

	case 0x81:
			C = res(0, C);
		break;

	case 0x82:
			D = res(0, D);
		break;

	case 0x83:
			E = res(0, E);
		break;

	case 0x84:
			H = res(0, H);
		break;

	case 0x85:
			L = res(0, L);
		break;

	case 0x86:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(0, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x87:
			A = res(0, A);
		break;

	case 0x88:
			B = res(1, B);
		break;

	case 0x89:
			C = res(1, C);
		break;

	case 0x8a:
			D = res(1, D);
		break;

	case 0x8b:
			E = res(1, E);
		break;

	case 0x8c:
			H = res(1, H);
		break;

	case 0x8d:
			L = res(1, L);
		break;

	case 0x8e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(1, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x8f:
			A = res(1, A);
		break;

	case 0x90:
			B = res(2, B);
		break;

	case 0x91:
			C = res(2, C);
		break;

	case 0x92:
			D = res(2, D);
		break;

	case 0x93:
			E = res(2, E);
		break;

	case 0x94:
			H = res(2, H);
		break;

	case 0x95:
			L = res(2, L);
		break;

	case 0x96:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(2, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x97:
			A = res(2, A);
		break;

	case 0x98:
			B = res(3, B);
		break;

	case 0x99:
			C = res(3, C);
		break;

	case 0x9a:
			D = res(3, D);
		break;

	case 0x9b:
			E = res(3, E);
		break;

	case 0x9c:
			H = res(3, H);
		break;

	case 0x9d:
			L = res(3, L);
		break;

	case 0x9e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(3, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x9f:
			A = res(3, A);
		break;

	case 0xa0:
			B = res(4, B);
		break;

	case 0xa1:
			C = res(4, C);
		break;

	case 0xa2:
			D = res(4, D);
		break;

	case 0xa3:
			E = res(4, E);
		break;

	case 0xa4:
			H = res(4, H);
		break;

	case 0xa5:
			L = res(4, L);
		break;

	case 0xa6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(4, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa7:
			A = res(4, A);
		break;

	case 0xa8:
			B = res(5, B);
		break;

	case 0xa9:
			C = res(5, C);
		break;

	case 0xaa:
			D = res(5, D);
		break;

	case 0xab:
			E = res(5, E);
		break;

	case 0xac:
			H = res(5, H);
		break;

	case 0xad:
			L = res(5, L);
		break;

	case 0xae:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(5, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xaf:
			A = res(5, A);
		break;

	case 0xb0:
			B = res(6, B);
		break;

	case 0xb1:
			C = res(6, C);
		break;

	case 0xb2:
			D = res(6, D);
		break;

	case 0xb3:
			E = res(6, E);
		break;

	case 0xb4:
			H = res(6, H);
		break;

	case 0xb5:
			L = res(6, L);
		break;

	case 0xb6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(6, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb7:
			A = res(6, A);
		break;

	case 0xb8:
			B = res(7, B);
		break;

	case 0xb9:
			C = res(7, C);
		break;

	case 0xba:
			D = res(7, D);
		break;

	case 0xbb:
			E = res(7, E);
		break;

	case 0xbc:
			H = res(7, H);
		break;

	case 0xbd:
			L = res(7, L);
		break;

	case 0xbe:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(7, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xbf:
			A = res(7, A);
		break;

	case 0xc0:
			B = set(0, B);
		break;

	case 0xc1:
			C = set(0, C);
		break;

	case 0xc2:
			D = set(0, D);
		break;

	case 0xc3:
			E = set(0, E);
		break;

	case 0xc4:
			H = set(0, H);
		break;

	case 0xc5:
			L = set(0, L);
		break;

	case 0xc6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(0, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc7:
			A = set(0, A);
		break;

	case 0xc8:
			B = set(1, B);
		break;

	case 0xc9:
			C = set(1, C);
		break;

	case 0xca:
			D = set(1, D);
		break;

	case 0xcb:
			E = set(1, E);
		break;

	case 0xcc:
			H = set(1, H);
		break;

	case 0xcd:
			L = set(1, L);
		break;

	case 0xce:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(1, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xcf:
			A = set(1, A);
		break;

	case 0xd0:
			B = set(2, B);
		break;

	case 0xd1:
			C = set(2, C);
		break;

	case 0xd2:
			D = set(2, D);
		break;

	case 0xd3:
			E = set(2, E);
		break;

	case 0xd4:
			H = set(2, H);
		break;

	case 0xd5:
			L = set(2, L);
		break;

	case 0xd6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(2, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd7:
			A = set(2, A);
		break;

	case 0xd8:
			B = set(3, B);
		break;

	case 0xd9:
			C = set(3, C);
		break;

	case 0xda:
			D = set(3, D);
		break;

	case 0xdb:
			E = set(3, E);
		break;

	case 0xdc:
			H = set(3, H);
		break;

	case 0xdd:
			L = set(3, L);
		break;

	case 0xde:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(3, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xdf:
			A = set(3, A);
		break;

	case 0xe0:
			B = set(4, B);
		break;

	case 0xe1:
			C = set(4, C);
		break;

	case 0xe2:
			D = set(4, D);
		break;

	case 0xe3:
			E = set(4, E);
		break;

	case 0xe4:
			H = set(4, H);
		break;

	case 0xe5:
			L = set(4, L);
		break;

	case 0xe6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(4, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe7:
			A = set(4, A);
		break;

	case 0xe8:
			B = set(5, B);
		break;

	case 0xe9:
			C = set(5, C);
		break;

	case 0xea:
			D = set(5, D);
		break;

	case 0xeb:
			E = set(5, E);
		break;

	case 0xec:
			H = set(5, H);
		break;

	case 0xed:
			L = set(5, L);
		break;

	case 0xee:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(5, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xef:
			A = set(5, A);
		break;

	case 0xf0:
			B = set(6, B);
		break;

	case 0xf1:
			C = set(6, C);
		break;

	case 0xf2:
			D = set(6, D);
		break;

	case 0xf3:
			E = set(6, E);
		break;

	case 0xf4:
			H = set(6, H);
		break;

	case 0xf5:
			L = set(6, L);
		break;

	case 0xf6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(6, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf7:
			A = set(6, A);
		break;

	case 0xf8:
			B = set(7, B);
		break;

	case 0xf9:
			C = set(7, C);
		break;

	case 0xfa:
			D = set(7, D);
		break;

	case 0xfb:
			E = set(7, E);
		break;

	case 0xfc:
			H = set(7, H);
		break;

	case 0xfd:
			L = set(7, L);
		break;

	case 0xfe:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(7, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xff:
			A = set(7, A);
		break;


	}
		break;

}
break; // prefix: 0xcb

case 0xfe:
{
	switch (u8(m_ref >> 8)) // opcode
	{
	case 0x00:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = rlc(TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x01:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = rlc(TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x02:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = rlc(TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x03:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = rlc(TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x04:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = rlc(TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x05:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = rlc(TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x06:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = rlc(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x07:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = rlc(TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x08:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = rrc(TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x09:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = rrc(TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x0a:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = rrc(TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x0b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = rrc(TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x0c:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = rrc(TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x0d:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = rrc(TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x0e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = rrc(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x0f:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = rrc(TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x10:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = rl(TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x11:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = rl(TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x12:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = rl(TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x13:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = rl(TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x14:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = rl(TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x15:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = rl(TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x16:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = rl(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x17:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = rl(TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x18:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = rr(TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x19:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = rr(TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x1a:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = rr(TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x1b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = rr(TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x1c:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = rr(TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x1d:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = rr(TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x1e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = rr(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x1f:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = rr(TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x20:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = sla(TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x21:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = sla(TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x22:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = sla(TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x23:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = sla(TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x24:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = sla(TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x25:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = sla(TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x26:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = sla(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x27:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = sla(TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x28:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = sra(TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x29:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = sra(TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x2a:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = sra(TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x2b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = sra(TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x2c:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = sra(TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x2d:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = sra(TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x2e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = sra(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x2f:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = sra(TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x30:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = sll(TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x31:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = sll(TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x32:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = sll(TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x33:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = sll(TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x34:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = sll(TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x35:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = sll(TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x36:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = sll(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x37:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = sll(TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x38:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = srl(TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x39:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = srl(TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x3a:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = srl(TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x3b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = srl(TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x3c:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = srl(TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x3d:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = srl(TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x3e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = srl(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x3f:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = srl(TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x40:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4600;
			if (true) return;
			break;

		}
		break;

	case 0x41:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4600;
			if (true) return;
			break;

		}
		break;

	case 0x42:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4600;
			if (true) return;
			break;

		}
		break;

	case 0x43:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4600;
			if (true) return;
			break;

		}
		break;

	case 0x44:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4600;
			if (true) return;
			break;

		}
		break;

	case 0x45:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4600;
			if (true) return;
			break;

		}
		break;

	case 0x46:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_xy(0, TDAT8);
			break;

		}
		break;

	case 0x47:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4600;
			if (true) return;
			break;

		}
		break;

	case 0x48:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4e00;
			if (true) return;
			break;

		}
		break;

	case 0x49:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4e00;
			if (true) return;
			break;

		}
		break;

	case 0x4a:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4e00;
			if (true) return;
			break;

		}
		break;

	case 0x4b:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4e00;
			if (true) return;
			break;

		}
		break;

	case 0x4c:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4e00;
			if (true) return;
			break;

		}
		break;

	case 0x4d:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4e00;
			if (true) return;
			break;

		}
		break;

	case 0x4e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_xy(1, TDAT8);
			break;

		}
		break;

	case 0x4f:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe4e00;
			if (true) return;
			break;

		}
		break;

	case 0x50:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5600;
			if (true) return;
			break;

		}
		break;

	case 0x51:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5600;
			if (true) return;
			break;

		}
		break;

	case 0x52:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5600;
			if (true) return;
			break;

		}
		break;

	case 0x53:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5600;
			if (true) return;
			break;

		}
		break;

	case 0x54:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5600;
			if (true) return;
			break;

		}
		break;

	case 0x55:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5600;
			if (true) return;
			break;

		}
		break;

	case 0x56:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_xy(2, TDAT8);
			break;

		}
		break;

	case 0x57:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5600;
			if (true) return;
			break;

		}
		break;

	case 0x58:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5e00;
			if (true) return;
			break;

		}
		break;

	case 0x59:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5e00;
			if (true) return;
			break;

		}
		break;

	case 0x5a:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5e00;
			if (true) return;
			break;

		}
		break;

	case 0x5b:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5e00;
			if (true) return;
			break;

		}
		break;

	case 0x5c:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5e00;
			if (true) return;
			break;

		}
		break;

	case 0x5d:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5e00;
			if (true) return;
			break;

		}
		break;

	case 0x5e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_xy(3, TDAT8);
			break;

		}
		break;

	case 0x5f:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe5e00;
			if (true) return;
			break;

		}
		break;

	case 0x60:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6600;
			if (true) return;
			break;

		}
		break;

	case 0x61:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6600;
			if (true) return;
			break;

		}
		break;

	case 0x62:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6600;
			if (true) return;
			break;

		}
		break;

	case 0x63:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6600;
			if (true) return;
			break;

		}
		break;

	case 0x64:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6600;
			if (true) return;
			break;

		}
		break;

	case 0x65:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6600;
			if (true) return;
			break;

		}
		break;

	case 0x66:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_xy(4, TDAT8);
			break;

		}
		break;

	case 0x67:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6600;
			if (true) return;
			break;

		}
		break;

	case 0x68:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6e00;
			if (true) return;
			break;

		}
		break;

	case 0x69:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6e00;
			if (true) return;
			break;

		}
		break;

	case 0x6a:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6e00;
			if (true) return;
			break;

		}
		break;

	case 0x6b:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6e00;
			if (true) return;
			break;

		}
		break;

	case 0x6c:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6e00;
			if (true) return;
			break;

		}
		break;

	case 0x6d:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6e00;
			if (true) return;
			break;

		}
		break;

	case 0x6e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_xy(5, TDAT8);
			break;

		}
		break;

	case 0x6f:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe6e00;
			if (true) return;
			break;

		}
		break;

	case 0x70:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7600;
			if (true) return;
			break;

		}
		break;

	case 0x71:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7600;
			if (true) return;
			break;

		}
		break;

	case 0x72:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7600;
			if (true) return;
			break;

		}
		break;

	case 0x73:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7600;
			if (true) return;
			break;

		}
		break;

	case 0x74:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7600;
			if (true) return;
			break;

		}
		break;

	case 0x75:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7600;
			if (true) return;
			break;

		}
		break;

	case 0x76:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_xy(6, TDAT8);
			break;

		}
		break;

	case 0x77:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7600;
			if (true) return;
			break;

		}
		break;

	case 0x78:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7e00;
			if (true) return;
			break;

		}
		break;

	case 0x79:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7e00;
			if (true) return;
			break;

		}
		break;

	case 0x7a:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7e00;
			if (true) return;
			break;

		}
		break;

	case 0x7b:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7e00;
			if (true) return;
			break;

		}
		break;

	case 0x7c:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7e00;
			if (true) return;
			break;

		}
		break;

	case 0x7d:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7e00;
			if (true) return;
			break;

		}
		break;

	case 0x7e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			bit_xy(7, TDAT8);
			break;

		}
		break;

	case 0x7f:
		switch (u8(m_ref))
		{
		case 0x00:
			m_ref = 0xfe7e00;
			if (true) return;
			break;

		}
		break;

	case 0x80:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = res(0, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x81:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = res(0, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x82:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = res(0, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x83:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = res(0, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x84:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = res(0, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x85:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = res(0, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x86:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(0, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x87:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = res(0, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x88:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = res(1, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x89:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = res(1, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x8a:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = res(1, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x8b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = res(1, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x8c:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = res(1, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x8d:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = res(1, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x8e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(1, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x8f:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = res(1, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x90:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = res(2, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x91:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = res(2, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x92:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = res(2, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x93:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = res(2, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x94:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = res(2, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x95:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = res(2, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x96:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(2, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x97:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = res(2, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x98:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = res(3, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x99:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = res(3, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x9a:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = res(3, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x9b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = res(3, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x9c:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = res(3, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x9d:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = res(3, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x9e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(3, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x9f:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = res(3, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa0:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = res(4, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa1:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = res(4, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa2:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = res(4, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa3:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = res(4, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa4:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = res(4, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa5:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = res(4, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(4, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa7:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = res(4, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa8:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = res(5, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xa9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = res(5, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xaa:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = res(5, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xab:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = res(5, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xac:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = res(5, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xad:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = res(5, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xae:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(5, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xaf:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = res(5, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb0:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = res(6, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb1:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = res(6, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb2:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = res(6, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb3:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = res(6, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb4:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = res(6, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb5:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = res(6, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(6, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb7:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = res(6, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb8:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = res(7, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xb9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = res(7, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xba:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = res(7, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xbb:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = res(7, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xbc:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = res(7, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xbd:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = res(7, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xbe:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = res(7, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xbf:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = res(7, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc0:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = set(0, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc1:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = set(0, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc2:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = set(0, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc3:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = set(0, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc4:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = set(0, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc5:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = set(0, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(0, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc7:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = set(0, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc8:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = set(1, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = set(1, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xca:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = set(1, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xcb:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = set(1, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xcc:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = set(1, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xcd:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = set(1, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xce:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(1, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xcf:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = set(1, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd0:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = set(2, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd1:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = set(2, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd2:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = set(2, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd3:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = set(2, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd4:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = set(2, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd5:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = set(2, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(2, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd7:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = set(2, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd8:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = set(3, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = set(3, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xda:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = set(3, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xdb:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = set(3, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xdc:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = set(3, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xdd:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = set(3, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xde:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(3, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xdf:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = set(3, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe0:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = set(4, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe1:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = set(4, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe2:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = set(4, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe3:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = set(4, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe4:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = set(4, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe5:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = set(4, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(4, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe7:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = set(4, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe8:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = set(5, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = set(5, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xea:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = set(5, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xeb:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = set(5, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xec:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = set(5, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xed:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = set(5, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xee:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(5, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xef:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = set(5, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf0:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = set(6, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf1:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = set(6, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf2:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = set(6, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf3:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = set(6, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf4:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = set(6, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf5:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = set(6, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(6, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf7:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = set(6, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf8:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B = set(7, TDAT8); TDAT8 = B;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			C = set(7, TDAT8); TDAT8 = C;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xfa:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			D = set(7, TDAT8); TDAT8 = D;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xfb:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			E = set(7, TDAT8); TDAT8 = E;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xfc:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			H = set(7, TDAT8); TDAT8 = H;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xfd:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			L = set(7, TDAT8); TDAT8 = L;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xfe:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			TDAT8 = set(7, TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xff:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = m_ea;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			A = set(7, TDAT8); TDAT8 = A;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;


	}
		break;

}
break; // prefix: 0xfe

case 0xdd:
{
	switch (u8(m_ref >> 8)) // opcode
	{
	case 0x00:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000000;
			if (true) return;
			break;

		}
		break;

	case 0x01:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000100;
			if (true) return;
			break;

		}
		break;

	case 0x02:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000200;
			if (true) return;
			break;

		}
		break;

	case 0x03:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000300;
			if (true) return;
			break;

		}
		break;

	case 0x04:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000400;
			if (true) return;
			break;

		}
		break;

	case 0x05:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000500;
			if (true) return;
			break;

		}
		break;

	case 0x06:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000600;
			if (true) return;
			break;

		}
		break;

	case 0x07:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000700;
			if (true) return;
			break;

		}
		break;

	case 0x08:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000800;
			if (true) return;
			break;

		}
		break;

	case 0x09:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IX; TDAT2 = BC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			IX = TDAT;
			break;

		}
		break;

	case 0x0a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000a00;
			if (true) return;
			break;

		}
		break;

	case 0x0b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000b00;
			if (true) return;
			break;

		}
		break;

	case 0x0c:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000c00;
			if (true) return;
			break;

		}
		break;

	case 0x0d:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000d00;
			if (true) return;
			break;

		}
		break;

	case 0x0e:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000e00;
			if (true) return;
			break;

		}
		break;

	case 0x0f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000f00;
			if (true) return;
			break;

		}
		break;

	case 0x10:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001000;
			if (true) return;
			break;

		}
		break;

	case 0x11:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001100;
			if (true) return;
			break;

		}
		break;

	case 0x12:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001200;
			if (true) return;
			break;

		}
		break;

	case 0x13:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001300;
			if (true) return;
			break;

		}
		break;

	case 0x14:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001400;
			if (true) return;
			break;

		}
		break;

	case 0x15:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001500;
			if (true) return;
			break;

		}
		break;

	case 0x16:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001600;
			if (true) return;
			break;

		}
		break;

	case 0x17:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001700;
			if (true) return;
			break;

		}
		break;

	case 0x18:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001800;
			if (true) return;
			break;

		}
		break;

	case 0x19:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IX; TDAT2 = DE;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			IX = TDAT;
			break;

		}
		break;

	case 0x1a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001a00;
			if (true) return;
			break;

		}
		break;

	case 0x1b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001b00;
			if (true) return;
			break;

		}
		break;

	case 0x1c:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001c00;
			if (true) return;
			break;

		}
		break;

	case 0x1d:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001d00;
			if (true) return;
			break;

		}
		break;

	case 0x1e:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001e00;
			if (true) return;
			break;

		}
		break;

	case 0x1f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001f00;
			if (true) return;
			break;

		}
		break;

	case 0x20:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x002000;
			if (true) return;
			break;

		}
		break;

	case 0x21:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			IX = TDAT;
			break;

		}
		break;

	case 0x22:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea; TDAT = IX;
			[[fallthrough]];
		case 0x05:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TADR++; TDAT8 = TDAT_H;
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			WZ = m_ea + 1;
			break;

		}
		break;

	case 0x23:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			IX++;
			break;

		}
		break;

	case 0x24:
			inc(HX);
		break;

	case 0x25:
			dec(HX);
		break;

	case 0x26:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			HX = TDAT8;
			break;

		}
		break;

	case 0x27:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x002700;
			if (true) return;
			break;

		}
		break;

	case 0x28:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x002800;
			if (true) return;
			break;

		}
		break;

	case 0x29:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IX; TDAT2 = IX;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			IX = TDAT;
			break;

		}
		break;

	case 0x2a:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea;
			[[fallthrough]];
		case 0x05:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TDAT_H = TDAT_L; TADR++;
			[[fallthrough]];
		case 0x07:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			using std::swap; std::swap(TDAT_H, TDAT_L);
			IX = TDAT; WZ = m_ea + 1;
			break;

		}
		break;

	case 0x2b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			IX--;
			break;

		}
		break;

	case 0x2c:
			inc(LX);
		break;

	case 0x2d:
			dec(LX);
		break;

	case 0x2e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			LX = TDAT8;
			break;

		}
		break;

	case 0x2f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x002f00;
			if (true) return;
			break;

		}
		break;

	case 0x30:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003000;
			if (true) return;
			break;

		}
		break;

	case 0x31:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003100;
			if (true) return;
			break;

		}
		break;

	case 0x32:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003200;
			if (true) return;
			break;

		}
		break;

	case 0x33:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003300;
			if (true) return;
			break;

		}
		break;

	case 0x34:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			inc(TDAT8);
			[[fallthrough]];
		case 0x0b:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0b;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0c;
				return;
			}
			[[fallthrough]];
		case 0x0c:
			break;

		}
		break;

	case 0x35:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			dec(TDAT8);
			[[fallthrough]];
		case 0x0b:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0b;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0c;
				return;
			}
			[[fallthrough]];
		case 0x0c:
			break;

		}
		break;

	case 0x36:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TADR = m_ea;
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			break;

		}
		break;

	case 0x37:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003700;
			if (true) return;
			break;

		}
		break;

	case 0x38:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003800;
			if (true) return;
			break;

		}
		break;

	case 0x39:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IX; TDAT2 = SP;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			IX = TDAT;
			break;

		}
		break;

	case 0x3a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003a00;
			if (true) return;
			break;

		}
		break;

	case 0x3b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003b00;
			if (true) return;
			break;

		}
		break;

	case 0x3c:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003c00;
			if (true) return;
			break;

		}
		break;

	case 0x3d:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003d00;
			if (true) return;
			break;

		}
		break;

	case 0x3e:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003e00;
			if (true) return;
			break;

		}
		break;

	case 0x3f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003f00;
			if (true) return;
			break;

		}
		break;

	case 0x40:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004000;
			if (true) return;
			break;

		}
		break;

	case 0x41:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004100;
			if (true) return;
			break;

		}
		break;

	case 0x42:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004200;
			if (true) return;
			break;

		}
		break;

	case 0x43:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004300;
			if (true) return;
			break;

		}
		break;

	case 0x44:
			B = HX;
		break;

	case 0x45:
			B = LX;
		break;

	case 0x46:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			B = TDAT8;
			break;

		}
		break;

	case 0x47:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004700;
			if (true) return;
			break;

		}
		break;

	case 0x48:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004800;
			if (true) return;
			break;

		}
		break;

	case 0x49:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004900;
			if (true) return;
			break;

		}
		break;

	case 0x4a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004a00;
			if (true) return;
			break;

		}
		break;

	case 0x4b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004b00;
			if (true) return;
			break;

		}
		break;

	case 0x4c:
			C = HX;
		break;

	case 0x4d:
			C = LX;
		break;

	case 0x4e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			C = TDAT8;
			break;

		}
		break;

	case 0x4f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004f00;
			if (true) return;
			break;

		}
		break;

	case 0x50:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005000;
			if (true) return;
			break;

		}
		break;

	case 0x51:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005100;
			if (true) return;
			break;

		}
		break;

	case 0x52:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005200;
			if (true) return;
			break;

		}
		break;

	case 0x53:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005300;
			if (true) return;
			break;

		}
		break;

	case 0x54:
			D = HX;
		break;

	case 0x55:
			D = LX;
		break;

	case 0x56:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			D = TDAT8;
			break;

		}
		break;

	case 0x57:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005700;
			if (true) return;
			break;

		}
		break;

	case 0x58:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005800;
			if (true) return;
			break;

		}
		break;

	case 0x59:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005900;
			if (true) return;
			break;

		}
		break;

	case 0x5a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005a00;
			if (true) return;
			break;

		}
		break;

	case 0x5b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005b00;
			if (true) return;
			break;

		}
		break;

	case 0x5c:
			E = HX;
		break;

	case 0x5d:
			E = LX;
		break;

	case 0x5e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			E = TDAT8;
			break;

		}
		break;

	case 0x5f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005f00;
			if (true) return;
			break;

		}
		break;

	case 0x60:
			HX = B;
		break;

	case 0x61:
			HX = C;
		break;

	case 0x62:
			HX = D;
		break;

	case 0x63:
			HX = E;
		break;

	case 0x64:
		break;

	case 0x65:
			HX = LX;
		break;

	case 0x66:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			H = TDAT8;
			break;

		}
		break;

	case 0x67:
			HX = A;
		break;

	case 0x68:
			LX = B;
		break;

	case 0x69:
			LX = C;
		break;

	case 0x6a:
			LX = D;
		break;

	case 0x6b:
			LX = E;
		break;

	case 0x6c:
			LX = HX;
		break;

	case 0x6d:
		break;

	case 0x6e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			L = TDAT8;
			break;

		}
		break;

	case 0x6f:
			LX = A;
		break;

	case 0x70:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = B;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x71:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = C;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x72:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = D;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x73:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = E;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x74:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = H;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x75:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = L;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x76:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007600;
			if (true) return;
			break;

		}
		break;

	case 0x77:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = A;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x78:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007800;
			if (true) return;
			break;

		}
		break;

	case 0x79:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007900;
			if (true) return;
			break;

		}
		break;

	case 0x7a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007a00;
			if (true) return;
			break;

		}
		break;

	case 0x7b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007b00;
			if (true) return;
			break;

		}
		break;

	case 0x7c:
			A = HX;
		break;

	case 0x7d:
			A = LX;
		break;

	case 0x7e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			A = TDAT8;
			break;

		}
		break;

	case 0x7f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007f00;
			if (true) return;
			break;

		}
		break;

	case 0x80:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008000;
			if (true) return;
			break;

		}
		break;

	case 0x81:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008100;
			if (true) return;
			break;

		}
		break;

	case 0x82:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008200;
			if (true) return;
			break;

		}
		break;

	case 0x83:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008300;
			if (true) return;
			break;

		}
		break;

	case 0x84:
			add_a(HX);
		break;

	case 0x85:
			add_a(LX);
		break;

	case 0x86:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			add_a(TDAT8);
			break;

		}
		break;

	case 0x87:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008700;
			if (true) return;
			break;

		}
		break;

	case 0x88:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008800;
			if (true) return;
			break;

		}
		break;

	case 0x89:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008900;
			if (true) return;
			break;

		}
		break;

	case 0x8a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008a00;
			if (true) return;
			break;

		}
		break;

	case 0x8b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008b00;
			if (true) return;
			break;

		}
		break;

	case 0x8c:
			adc_a(HX);
		break;

	case 0x8d:
			adc_a(LX);
		break;

	case 0x8e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			adc_a(TDAT8);
			break;

		}
		break;

	case 0x8f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008f00;
			if (true) return;
			break;

		}
		break;

	case 0x90:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009000;
			if (true) return;
			break;

		}
		break;

	case 0x91:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009100;
			if (true) return;
			break;

		}
		break;

	case 0x92:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009200;
			if (true) return;
			break;

		}
		break;

	case 0x93:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009300;
			if (true) return;
			break;

		}
		break;

	case 0x94:
			sub(HX);
		break;

	case 0x95:
			sub(LX);
		break;

	case 0x96:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			sub(TDAT8);
			break;

		}
		break;

	case 0x97:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009700;
			if (true) return;
			break;

		}
		break;

	case 0x98:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009800;
			if (true) return;
			break;

		}
		break;

	case 0x99:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009900;
			if (true) return;
			break;

		}
		break;

	case 0x9a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009a00;
			if (true) return;
			break;

		}
		break;

	case 0x9b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009b00;
			if (true) return;
			break;

		}
		break;

	case 0x9c:
			sbc_a(HX);
		break;

	case 0x9d:
			sbc_a(LX);
		break;

	case 0x9e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			sbc_a(TDAT8);
			break;

		}
		break;

	case 0x9f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009f00;
			if (true) return;
			break;

		}
		break;

	case 0xa0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a000;
			if (true) return;
			break;

		}
		break;

	case 0xa1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a100;
			if (true) return;
			break;

		}
		break;

	case 0xa2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a200;
			if (true) return;
			break;

		}
		break;

	case 0xa3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a300;
			if (true) return;
			break;

		}
		break;

	case 0xa4:
			and_a(HX);
		break;

	case 0xa5:
			and_a(LX);
		break;

	case 0xa6:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			and_a(TDAT8);
			break;

		}
		break;

	case 0xa7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a700;
			if (true) return;
			break;

		}
		break;

	case 0xa8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a800;
			if (true) return;
			break;

		}
		break;

	case 0xa9:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a900;
			if (true) return;
			break;

		}
		break;

	case 0xaa:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00aa00;
			if (true) return;
			break;

		}
		break;

	case 0xab:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ab00;
			if (true) return;
			break;

		}
		break;

	case 0xac:
			xor_a(HX);
		break;

	case 0xad:
			xor_a(LX);
		break;

	case 0xae:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			xor_a(TDAT8);
			break;

		}
		break;

	case 0xaf:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00af00;
			if (true) return;
			break;

		}
		break;

	case 0xb0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b000;
			if (true) return;
			break;

		}
		break;

	case 0xb1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b100;
			if (true) return;
			break;

		}
		break;

	case 0xb2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b200;
			if (true) return;
			break;

		}
		break;

	case 0xb3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b300;
			if (true) return;
			break;

		}
		break;

	case 0xb4:
			or_a(HX);
		break;

	case 0xb5:
			or_a(LX);
		break;

	case 0xb6:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			or_a(TDAT8);
			break;

		}
		break;

	case 0xb7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b700;
			if (true) return;
			break;

		}
		break;

	case 0xb8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b800;
			if (true) return;
			break;

		}
		break;

	case 0xb9:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b900;
			if (true) return;
			break;

		}
		break;

	case 0xba:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ba00;
			if (true) return;
			break;

		}
		break;

	case 0xbb:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00bb00;
			if (true) return;
			break;

		}
		break;

	case 0xbc:
			cp(HX);
		break;

	case 0xbd:
			cp(LX);
		break;

	case 0xbe:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			cp(TDAT8);
			break;

		}
		break;

	case 0xbf:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00bf00;
			if (true) return;
			break;

		}
		break;

	case 0xc0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c000;
			if (true) return;
			break;

		}
		break;

	case 0xc1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c100;
			if (true) return;
			break;

		}
		break;

	case 0xc2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c200;
			if (true) return;
			break;

		}
		break;

	case 0xc3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c300;
			if (true) return;
			break;

		}
		break;

	case 0xc4:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c400;
			if (true) return;
			break;

		}
		break;

	case 0xc5:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c500;
			if (true) return;
			break;

		}
		break;

	case 0xc6:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c600;
			if (true) return;
			break;

		}
		break;

	case 0xc7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c700;
			if (true) return;
			break;

		}
		break;

	case 0xc8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c800;
			if (true) return;
			break;

		}
		break;

	case 0xc9:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c900;
			if (true) return;
			break;

		}
		break;

	case 0xca:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ca00;
			if (true) return;
			break;

		}
		break;

	case 0xcb:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IX + (s8)TDAT8); WZ = m_ea;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_ref = (0xfe << 16) | (TDAT8 << 8);
			if (true) return;
			break;

		}
		break;

	case 0xcc:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00cc00;
			if (true) return;
			break;

		}
		break;

	case 0xcd:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00cd00;
			if (true) return;
			break;

		}
		break;

	case 0xce:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ce00;
			if (true) return;
			break;

		}
		break;

	case 0xcf:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00cf00;
			if (true) return;
			break;

		}
		break;

	case 0xd0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d000;
			if (true) return;
			break;

		}
		break;

	case 0xd1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d100;
			if (true) return;
			break;

		}
		break;

	case 0xd2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d200;
			if (true) return;
			break;

		}
		break;

	case 0xd3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d300;
			if (true) return;
			break;

		}
		break;

	case 0xd4:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d400;
			if (true) return;
			break;

		}
		break;

	case 0xd5:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d500;
			if (true) return;
			break;

		}
		break;

	case 0xd6:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d600;
			if (true) return;
			break;

		}
		break;

	case 0xd7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d700;
			if (true) return;
			break;

		}
		break;

	case 0xd8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d800;
			if (true) return;
			break;

		}
		break;

	case 0xd9:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d900;
			if (true) return;
			break;

		}
		break;

	case 0xda:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00da00;
			if (true) return;
			break;

		}
		break;

	case 0xdb:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00db00;
			if (true) return;
			break;

		}
		break;

	case 0xdc:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00dc00;
			if (true) return;
			break;

		}
		break;

	case 0xdd:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00dd00;
			if (true) return;
			break;

		}
		break;

	case 0xde:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00de00;
			if (true) return;
			break;

		}
		break;

	case 0xdf:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00df00;
			if (true) return;
			break;

		}
		break;

	case 0xe0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e000;
			if (true) return;
			break;

		}
		break;

	case 0xe1:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			IX = TDAT;
			break;

		}
		break;

	case 0xe2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e200;
			if (true) return;
			break;

		}
		break;

	case 0xe3:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IX;
			TDAT2 = TDAT;
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			TADR = SP-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			using std::swap; std::swap(TDAT, TDAT2);
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			TADR = SP;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			using std::swap; std::swap(TDAT, TDAT2); WZ = TDAT;
			IX = TDAT;
			break;

		}
		break;

	case 0xe4:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e400;
			if (true) return;
			break;

		}
		break;

	case 0xe5:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IX;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe6:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e600;
			if (true) return;
			break;

		}
		break;

	case 0xe7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e700;
			if (true) return;
			break;

		}
		break;

	case 0xe8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e800;
			if (true) return;
			break;

		}
		break;

	case 0xe9:
			PC = IX;
		break;

	case 0xea:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ea00;
			if (true) return;
			break;

		}
		break;

	case 0xeb:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00eb00;
			if (true) return;
			break;

		}
		break;

	case 0xec:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ec00;
			if (true) return;
			break;

		}
		break;

	case 0xed:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ed00;
			if (true) return;
			break;

		}
		break;

	case 0xee:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ee00;
			if (true) return;
			break;

		}
		break;

	case 0xef:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ef00;
			if (true) return;
			break;

		}
		break;

	case 0xf0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f000;
			if (true) return;
			break;

		}
		break;

	case 0xf1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f100;
			if (true) return;
			break;

		}
		break;

	case 0xf2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f200;
			if (true) return;
			break;

		}
		break;

	case 0xf3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f300;
			if (true) return;
			break;

		}
		break;

	case 0xf4:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f400;
			if (true) return;
			break;

		}
		break;

	case 0xf5:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f500;
			if (true) return;
			break;

		}
		break;

	case 0xf6:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f600;
			if (true) return;
			break;

		}
		break;

	case 0xf7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f700;
			if (true) return;
			break;

		}
		break;

	case 0xf8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f800;
			if (true) return;
			break;

		}
		break;

	case 0xf9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP = IX;
			break;

		}
		break;

	case 0xfa:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fa00;
			if (true) return;
			break;

		}
		break;

	case 0xfb:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fb00;
			if (true) return;
			break;

		}
		break;

	case 0xfc:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fc00;
			if (true) return;
			break;

		}
		break;

	case 0xfd:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fd00;
			if (true) return;
			break;

		}
		break;

	case 0xfe:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fe00;
			if (true) return;
			break;

		}
		break;

	case 0xff:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ff00;
			if (true) return;
			break;

		}
		break;


	}
		break;

}
break; // prefix: 0xdd

case 0xfd:
{
	switch (u8(m_ref >> 8)) // opcode
	{
	case 0x00:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000000;
			if (true) return;
			break;

		}
		break;

	case 0x01:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000100;
			if (true) return;
			break;

		}
		break;

	case 0x02:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000200;
			if (true) return;
			break;

		}
		break;

	case 0x03:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000300;
			if (true) return;
			break;

		}
		break;

	case 0x04:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000400;
			if (true) return;
			break;

		}
		break;

	case 0x05:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000500;
			if (true) return;
			break;

		}
		break;

	case 0x06:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000600;
			if (true) return;
			break;

		}
		break;

	case 0x07:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000700;
			if (true) return;
			break;

		}
		break;

	case 0x08:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000800;
			if (true) return;
			break;

		}
		break;

	case 0x09:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IY; TDAT2 = BC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			IY = TDAT;
			break;

		}
		break;

	case 0x0a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000a00;
			if (true) return;
			break;

		}
		break;

	case 0x0b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000b00;
			if (true) return;
			break;

		}
		break;

	case 0x0c:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000c00;
			if (true) return;
			break;

		}
		break;

	case 0x0d:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000d00;
			if (true) return;
			break;

		}
		break;

	case 0x0e:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000e00;
			if (true) return;
			break;

		}
		break;

	case 0x0f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x000f00;
			if (true) return;
			break;

		}
		break;

	case 0x10:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001000;
			if (true) return;
			break;

		}
		break;

	case 0x11:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001100;
			if (true) return;
			break;

		}
		break;

	case 0x12:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001200;
			if (true) return;
			break;

		}
		break;

	case 0x13:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001300;
			if (true) return;
			break;

		}
		break;

	case 0x14:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001400;
			if (true) return;
			break;

		}
		break;

	case 0x15:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001500;
			if (true) return;
			break;

		}
		break;

	case 0x16:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001600;
			if (true) return;
			break;

		}
		break;

	case 0x17:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001700;
			if (true) return;
			break;

		}
		break;

	case 0x18:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001800;
			if (true) return;
			break;

		}
		break;

	case 0x19:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IY; TDAT2 = DE;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			IY = TDAT;
			break;

		}
		break;

	case 0x1a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001a00;
			if (true) return;
			break;

		}
		break;

	case 0x1b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001b00;
			if (true) return;
			break;

		}
		break;

	case 0x1c:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001c00;
			if (true) return;
			break;

		}
		break;

	case 0x1d:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001d00;
			if (true) return;
			break;

		}
		break;

	case 0x1e:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001e00;
			if (true) return;
			break;

		}
		break;

	case 0x1f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x001f00;
			if (true) return;
			break;

		}
		break;

	case 0x20:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x002000;
			if (true) return;
			break;

		}
		break;

	case 0x21:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			IY = TDAT;
			break;

		}
		break;

	case 0x22:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea; TDAT = IY;
			[[fallthrough]];
		case 0x05:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TADR++; TDAT8 = TDAT_H;
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			WZ = m_ea + 1;
			break;

		}
		break;

	case 0x23:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			IY++;
			break;

		}
		break;

	case 0x24:
			inc(HY);
		break;

	case 0x25:
			dec(HY);
		break;

	case 0x26:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			HY = TDAT8;
			break;

		}
		break;

	case 0x27:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x002700;
			if (true) return;
			break;

		}
		break;

	case 0x28:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x002800;
			if (true) return;
			break;

		}
		break;

	case 0x29:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IY; TDAT2 = IY;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			IY = TDAT;
			break;

		}
		break;

	case 0x2a:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea;
			[[fallthrough]];
		case 0x05:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TDAT_H = TDAT_L; TADR++;
			[[fallthrough]];
		case 0x07:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			using std::swap; std::swap(TDAT_H, TDAT_L);
			IY = TDAT; WZ = m_ea + 1;
			break;

		}
		break;

	case 0x2b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			IY--;
			break;

		}
		break;

	case 0x2c:
			inc(LY);
		break;

	case 0x2d:
			dec(LY);
		break;

	case 0x2e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			LY = TDAT8;
			break;

		}
		break;

	case 0x2f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x002f00;
			if (true) return;
			break;

		}
		break;

	case 0x30:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003000;
			if (true) return;
			break;

		}
		break;

	case 0x31:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003100;
			if (true) return;
			break;

		}
		break;

	case 0x32:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003200;
			if (true) return;
			break;

		}
		break;

	case 0x33:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003300;
			if (true) return;
			break;

		}
		break;

	case 0x34:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			inc(TDAT8);
			[[fallthrough]];
		case 0x0b:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0b;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0c;
				return;
			}
			[[fallthrough]];
		case 0x0c:
			break;

		}
		break;

	case 0x35:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			dec(TDAT8);
			[[fallthrough]];
		case 0x0b:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0b;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0c;
				return;
			}
			[[fallthrough]];
		case 0x0c:
			break;

		}
		break;

	case 0x36:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TADR = m_ea;
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			break;

		}
		break;

	case 0x37:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003700;
			if (true) return;
			break;

		}
		break;

	case 0x38:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003800;
			if (true) return;
			break;

		}
		break;

	case 0x39:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IY; TDAT2 = SP;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			IY = TDAT;
			break;

		}
		break;

	case 0x3a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003a00;
			if (true) return;
			break;

		}
		break;

	case 0x3b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003b00;
			if (true) return;
			break;

		}
		break;

	case 0x3c:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003c00;
			if (true) return;
			break;

		}
		break;

	case 0x3d:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003d00;
			if (true) return;
			break;

		}
		break;

	case 0x3e:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003e00;
			if (true) return;
			break;

		}
		break;

	case 0x3f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x003f00;
			if (true) return;
			break;

		}
		break;

	case 0x40:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004000;
			if (true) return;
			break;

		}
		break;

	case 0x41:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004100;
			if (true) return;
			break;

		}
		break;

	case 0x42:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004200;
			if (true) return;
			break;

		}
		break;

	case 0x43:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004300;
			if (true) return;
			break;

		}
		break;

	case 0x44:
			B = HY;
		break;

	case 0x45:
			B = LY;
		break;

	case 0x46:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			B = TDAT8;
			break;

		}
		break;

	case 0x47:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004700;
			if (true) return;
			break;

		}
		break;

	case 0x48:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004800;
			if (true) return;
			break;

		}
		break;

	case 0x49:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004900;
			if (true) return;
			break;

		}
		break;

	case 0x4a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004a00;
			if (true) return;
			break;

		}
		break;

	case 0x4b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004b00;
			if (true) return;
			break;

		}
		break;

	case 0x4c:
			C = HY;
		break;

	case 0x4d:
			C = LY;
		break;

	case 0x4e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			C = TDAT8;
			break;

		}
		break;

	case 0x4f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x004f00;
			if (true) return;
			break;

		}
		break;

	case 0x50:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005000;
			if (true) return;
			break;

		}
		break;

	case 0x51:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005100;
			if (true) return;
			break;

		}
		break;

	case 0x52:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005200;
			if (true) return;
			break;

		}
		break;

	case 0x53:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005300;
			if (true) return;
			break;

		}
		break;

	case 0x54:
			D = HY;
		break;

	case 0x55:
			D = LY;
		break;

	case 0x56:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			D = TDAT8;
			break;

		}
		break;

	case 0x57:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005700;
			if (true) return;
			break;

		}
		break;

	case 0x58:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005800;
			if (true) return;
			break;

		}
		break;

	case 0x59:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005900;
			if (true) return;
			break;

		}
		break;

	case 0x5a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005a00;
			if (true) return;
			break;

		}
		break;

	case 0x5b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005b00;
			if (true) return;
			break;

		}
		break;

	case 0x5c:
			E = HY;
		break;

	case 0x5d:
			E = LY;
		break;

	case 0x5e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			E = TDAT8;
			break;

		}
		break;

	case 0x5f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x005f00;
			if (true) return;
			break;

		}
		break;

	case 0x60:
			HY = B;
		break;

	case 0x61:
			HY = C;
		break;

	case 0x62:
			HY = D;
		break;

	case 0x63:
			HY = E;
		break;

	case 0x64:
		break;

	case 0x65:
			HY = LY;
		break;

	case 0x66:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			H = TDAT8;
			break;

		}
		break;

	case 0x67:
			HY = A;
		break;

	case 0x68:
			LY = B;
		break;

	case 0x69:
			LY = C;
		break;

	case 0x6a:
			LY = D;
		break;

	case 0x6b:
			LY = E;
		break;

	case 0x6c:
			LY = HY;
		break;

	case 0x6d:
		break;

	case 0x6e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			L = TDAT8;
			break;

		}
		break;

	case 0x6f:
			LY = A;
		break;

	case 0x70:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = B;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x71:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = C;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x72:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = D;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x73:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = E;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x74:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = H;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x75:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = L;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x76:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007600;
			if (true) return;
			break;

		}
		break;

	case 0x77:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea; TDAT8 = A;
			[[fallthrough]];
		case 0x08:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x78:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007800;
			if (true) return;
			break;

		}
		break;

	case 0x79:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007900;
			if (true) return;
			break;

		}
		break;

	case 0x7a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007a00;
			if (true) return;
			break;

		}
		break;

	case 0x7b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007b00;
			if (true) return;
			break;

		}
		break;

	case 0x7c:
			A = HY;
		break;

	case 0x7d:
			A = LY;
		break;

	case 0x7e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			A = TDAT8;
			break;

		}
		break;

	case 0x7f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x007f00;
			if (true) return;
			break;

		}
		break;

	case 0x80:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008000;
			if (true) return;
			break;

		}
		break;

	case 0x81:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008100;
			if (true) return;
			break;

		}
		break;

	case 0x82:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008200;
			if (true) return;
			break;

		}
		break;

	case 0x83:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008300;
			if (true) return;
			break;

		}
		break;

	case 0x84:
			add_a(HY);
		break;

	case 0x85:
			add_a(LY);
		break;

	case 0x86:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			add_a(TDAT8);
			break;

		}
		break;

	case 0x87:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008700;
			if (true) return;
			break;

		}
		break;

	case 0x88:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008800;
			if (true) return;
			break;

		}
		break;

	case 0x89:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008900;
			if (true) return;
			break;

		}
		break;

	case 0x8a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008a00;
			if (true) return;
			break;

		}
		break;

	case 0x8b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008b00;
			if (true) return;
			break;

		}
		break;

	case 0x8c:
			adc_a(HY);
		break;

	case 0x8d:
			adc_a(LY);
		break;

	case 0x8e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			adc_a(TDAT8);
			break;

		}
		break;

	case 0x8f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x008f00;
			if (true) return;
			break;

		}
		break;

	case 0x90:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009000;
			if (true) return;
			break;

		}
		break;

	case 0x91:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009100;
			if (true) return;
			break;

		}
		break;

	case 0x92:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009200;
			if (true) return;
			break;

		}
		break;

	case 0x93:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009300;
			if (true) return;
			break;

		}
		break;

	case 0x94:
			sub(HY);
		break;

	case 0x95:
			sub(LY);
		break;

	case 0x96:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			sub(TDAT8);
			break;

		}
		break;

	case 0x97:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009700;
			if (true) return;
			break;

		}
		break;

	case 0x98:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009800;
			if (true) return;
			break;

		}
		break;

	case 0x99:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009900;
			if (true) return;
			break;

		}
		break;

	case 0x9a:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009a00;
			if (true) return;
			break;

		}
		break;

	case 0x9b:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009b00;
			if (true) return;
			break;

		}
		break;

	case 0x9c:
			sbc_a(HY);
		break;

	case 0x9d:
			sbc_a(LY);
		break;

	case 0x9e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			sbc_a(TDAT8);
			break;

		}
		break;

	case 0x9f:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x009f00;
			if (true) return;
			break;

		}
		break;

	case 0xa0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a000;
			if (true) return;
			break;

		}
		break;

	case 0xa1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a100;
			if (true) return;
			break;

		}
		break;

	case 0xa2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a200;
			if (true) return;
			break;

		}
		break;

	case 0xa3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a300;
			if (true) return;
			break;

		}
		break;

	case 0xa4:
			and_a(HY);
		break;

	case 0xa5:
			and_a(LY);
		break;

	case 0xa6:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			and_a(TDAT8);
			break;

		}
		break;

	case 0xa7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a700;
			if (true) return;
			break;

		}
		break;

	case 0xa8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a800;
			if (true) return;
			break;

		}
		break;

	case 0xa9:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00a900;
			if (true) return;
			break;

		}
		break;

	case 0xaa:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00aa00;
			if (true) return;
			break;

		}
		break;

	case 0xab:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ab00;
			if (true) return;
			break;

		}
		break;

	case 0xac:
			xor_a(HY);
		break;

	case 0xad:
			xor_a(LY);
		break;

	case 0xae:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			xor_a(TDAT8);
			break;

		}
		break;

	case 0xaf:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00af00;
			if (true) return;
			break;

		}
		break;

	case 0xb0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b000;
			if (true) return;
			break;

		}
		break;

	case 0xb1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b100;
			if (true) return;
			break;

		}
		break;

	case 0xb2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b200;
			if (true) return;
			break;

		}
		break;

	case 0xb3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b300;
			if (true) return;
			break;

		}
		break;

	case 0xb4:
			or_a(HY);
		break;

	case 0xb5:
			or_a(LY);
		break;

	case 0xb6:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			or_a(TDAT8);
			break;

		}
		break;

	case 0xb7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b700;
			if (true) return;
			break;

		}
		break;

	case 0xb8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b800;
			if (true) return;
			break;

		}
		break;

	case 0xb9:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00b900;
			if (true) return;
			break;

		}
		break;

	case 0xba:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ba00;
			if (true) return;
			break;

		}
		break;

	case 0xbb:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00bb00;
			if (true) return;
			break;

		}
		break;

	case 0xbc:
			cp(HY);
		break;

	case 0xbd:
			cp(LY);
		break;

	case 0xbe:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			TADR = m_ea;
			[[fallthrough]];
		case 0x08:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			cp(TDAT8);
			break;

		}
		break;

	case 0xbf:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00bf00;
			if (true) return;
			break;

		}
		break;

	case 0xc0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c000;
			if (true) return;
			break;

		}
		break;

	case 0xc1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c100;
			if (true) return;
			break;

		}
		break;

	case 0xc2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c200;
			if (true) return;
			break;

		}
		break;

	case 0xc3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c300;
			if (true) return;
			break;

		}
		break;

	case 0xc4:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c400;
			if (true) return;
			break;

		}
		break;

	case 0xc5:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c500;
			if (true) return;
			break;

		}
		break;

	case 0xc6:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c600;
			if (true) return;
			break;

		}
		break;

	case 0xc7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c700;
			if (true) return;
			break;

		}
		break;

	case 0xc8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c800;
			if (true) return;
			break;

		}
		break;

	case 0xc9:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00c900;
			if (true) return;
			break;

		}
		break;

	case 0xca:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ca00;
			if (true) return;
			break;

		}
		break;

	case 0xcb:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_ea = (u16)(IY + (s8)TDAT8); WZ = m_ea;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_ref = (0xfe << 16) | (TDAT8 << 8);
			if (true) return;
			break;

		}
		break;

	case 0xcc:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00cc00;
			if (true) return;
			break;

		}
		break;

	case 0xcd:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00cd00;
			if (true) return;
			break;

		}
		break;

	case 0xce:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ce00;
			if (true) return;
			break;

		}
		break;

	case 0xcf:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00cf00;
			if (true) return;
			break;

		}
		break;

	case 0xd0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d000;
			if (true) return;
			break;

		}
		break;

	case 0xd1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d100;
			if (true) return;
			break;

		}
		break;

	case 0xd2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d200;
			if (true) return;
			break;

		}
		break;

	case 0xd3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d300;
			if (true) return;
			break;

		}
		break;

	case 0xd4:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d400;
			if (true) return;
			break;

		}
		break;

	case 0xd5:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d500;
			if (true) return;
			break;

		}
		break;

	case 0xd6:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d600;
			if (true) return;
			break;

		}
		break;

	case 0xd7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d700;
			if (true) return;
			break;

		}
		break;

	case 0xd8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d800;
			if (true) return;
			break;

		}
		break;

	case 0xd9:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00d900;
			if (true) return;
			break;

		}
		break;

	case 0xda:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00da00;
			if (true) return;
			break;

		}
		break;

	case 0xdb:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00db00;
			if (true) return;
			break;

		}
		break;

	case 0xdc:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00dc00;
			if (true) return;
			break;

		}
		break;

	case 0xdd:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00dd00;
			if (true) return;
			break;

		}
		break;

	case 0xde:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00de00;
			if (true) return;
			break;

		}
		break;

	case 0xdf:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00df00;
			if (true) return;
			break;

		}
		break;

	case 0xe0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e000;
			if (true) return;
			break;

		}
		break;

	case 0xe1:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			IY = TDAT;
			break;

		}
		break;

	case 0xe2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e200;
			if (true) return;
			break;

		}
		break;

	case 0xe3:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IY;
			TDAT2 = TDAT;
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			TADR = SP-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			using std::swap; std::swap(TDAT, TDAT2);
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			TADR = SP;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			using std::swap; std::swap(TDAT, TDAT2); WZ = TDAT;
			IY = TDAT;
			break;

		}
		break;

	case 0xe4:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e400;
			if (true) return;
			break;

		}
		break;

	case 0xe5:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = IY;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe6:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e600;
			if (true) return;
			break;

		}
		break;

	case 0xe7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e700;
			if (true) return;
			break;

		}
		break;

	case 0xe8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00e800;
			if (true) return;
			break;

		}
		break;

	case 0xe9:
			PC = IY;
		break;

	case 0xea:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ea00;
			if (true) return;
			break;

		}
		break;

	case 0xeb:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00eb00;
			if (true) return;
			break;

		}
		break;

	case 0xec:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ec00;
			if (true) return;
			break;

		}
		break;

	case 0xed:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ed00;
			if (true) return;
			break;

		}
		break;

	case 0xee:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ee00;
			if (true) return;
			break;

		}
		break;

	case 0xef:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ef00;
			if (true) return;
			break;

		}
		break;

	case 0xf0:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f000;
			if (true) return;
			break;

		}
		break;

	case 0xf1:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f100;
			if (true) return;
			break;

		}
		break;

	case 0xf2:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f200;
			if (true) return;
			break;

		}
		break;

	case 0xf3:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f300;
			if (true) return;
			break;

		}
		break;

	case 0xf4:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f400;
			if (true) return;
			break;

		}
		break;

	case 0xf5:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f500;
			if (true) return;
			break;

		}
		break;

	case 0xf6:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f600;
			if (true) return;
			break;

		}
		break;

	case 0xf7:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f700;
			if (true) return;
			break;

		}
		break;

	case 0xf8:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00f800;
			if (true) return;
			break;

		}
		break;

	case 0xf9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP = IY;
			break;

		}
		break;

	case 0xfa:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fa00;
			if (true) return;
			break;

		}
		break;

	case 0xfb:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fb00;
			if (true) return;
			break;

		}
		break;

	case 0xfc:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fc00;
			if (true) return;
			break;

		}
		break;

	case 0xfd:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fd00;
			if (true) return;
			break;

		}
		break;

	case 0xfe:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00fe00;
			if (true) return;
			break;

		}
		break;

	case 0xff:
		switch (u8(m_ref))
		{
		case 0x00:
			illegal_1();
			m_ref = 0x00ff00;
			if (true) return;
			break;

		}
		break;


	}
		break;

}
break; // prefix: 0xfd

case 0xed:
{
	switch (u8(m_ref >> 8)) // opcode
	{
	case 0x00:
			illegal_2();
		break;

	case 0x01:
			illegal_2();
		break;

	case 0x02:
			illegal_2();
		break;

	case 0x03:
			illegal_2();
		break;

	case 0x04:
			illegal_2();
		break;

	case 0x05:
			illegal_2();
		break;

	case 0x06:
			illegal_2();
		break;

	case 0x07:
			illegal_2();
		break;

	case 0x08:
			illegal_2();
		break;

	case 0x09:
			illegal_2();
		break;

	case 0x0a:
			illegal_2();
		break;

	case 0x0b:
			illegal_2();
		break;

	case 0x0c:
			illegal_2();
		break;

	case 0x0d:
			illegal_2();
		break;

	case 0x0e:
			illegal_2();
		break;

	case 0x0f:
			illegal_2();
		break;

	case 0x10:
			illegal_2();
		break;

	case 0x11:
			illegal_2();
		break;

	case 0x12:
			illegal_2();
		break;

	case 0x13:
			illegal_2();
		break;

	case 0x14:
			illegal_2();
		break;

	case 0x15:
			illegal_2();
		break;

	case 0x16:
			illegal_2();
		break;

	case 0x17:
			illegal_2();
		break;

	case 0x18:
			illegal_2();
		break;

	case 0x19:
			illegal_2();
		break;

	case 0x1a:
			illegal_2();
		break;

	case 0x1b:
			illegal_2();
		break;

	case 0x1c:
			illegal_2();
		break;

	case 0x1d:
			illegal_2();
		break;

	case 0x1e:
			illegal_2();
		break;

	case 0x1f:
			illegal_2();
		break;

	case 0x20:
			illegal_2();
		break;

	case 0x21:
			illegal_2();
		break;

	case 0x22:
			illegal_2();
		break;

	case 0x23:
			A = (A << 4) | (A >> 4);
		break;

	case 0x24:
			A = bitswap<8>(A, 0, 1, 2, 3, 4, 5, 6, 7);
		break;

	case 0x25:
			illegal_2();
		break;

	case 0x26:
			illegal_2();
		break;

	case 0x27:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT_H = A;
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			and_a(TDAT8);
			A = TDAT_H;
			break;

		}
		break;

	case 0x28:
			DE <<= std::min(B & 31, 16);
		break;

	case 0x29:
		switch (u8(m_ref))
		{
		case 0x00:
			{
				const u16 fill = (DE & 0x8000) ? ~u16(0) : u16(0);
				DE = (DE >> std::min(B & 31, 16)) | (fill << (16 - std::min(B & 31, 16)));
			}
			break;

		}
		break;

	case 0x2a:
			DE >>= std::min(B & 31, 16);
		break;

	case 0x2b:
			DE = (DE >> std::min(B & 31, 16)) | (~u16(0) << (16 - std::min(B & 31, 16)));
		break;

	case 0x2c:
			DE = (DE << (B & 15)) | (DE >> (16 - (B & 15)));
		break;

	case 0x2d:
			illegal_2();
		break;

	case 0x2e:
			illegal_2();
		break;

	case 0x2f:
			illegal_2();
		break;

	case 0x30:
			DE = D * E;
		break;

	case 0x31:
			HL += A;
		break;

	case 0x32:
			DE += A;
		break;

	case 0x33:
			BC += A;
		break;

	case 0x34:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			HL += TDAT;
			break;

		}
		break;

	case 0x35:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			DE += TDAT;
			break;

		}
		break;

	case 0x36:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			BC += TDAT;
			break;

		}
		break;

	case 0x37:
			illegal_2();
		break;

	case 0x38:
			illegal_2();
		break;

	case 0x39:
			illegal_2();
		break;

	case 0x3a:
			illegal_2();
		break;

	case 0x3b:
			illegal_2();
		break;

	case 0x3c:
			illegal_2();
		break;

	case 0x3d:
			illegal_2();
		break;

	case 0x3e:
			illegal_2();
		break;

	case 0x3f:
			illegal_2();
		break;

	case 0x40:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			B = TDAT8; set_f((F & CF) | SZP[B]); WZ = TADR + 1;
			break;

		}
		break;

	case 0x41:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC; TDAT8 = B;
			[[fallthrough]];
		case 0x01:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = TADR + 1;
			break;

		}
		break;

	case 0x42:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = BC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = HL - TDAT - (F & CF); WZ = HL + 1; set_f((((HL ^ res ^ TDAT) >> 8) & HF) | NF | ((res >> 16) & CF) | ((res >> 8) & (SF | YF | XF)) | ((res & 0xffff) ? 0 : ZF) | (((TDAT ^ HL) & (HL ^ res) &0x8000) >> 13)); HL = (u16)res; }
			break;

		}
		break;

	case 0x43:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea; TDAT = BC;
			[[fallthrough]];
		case 0x05:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TADR++; TDAT8 = TDAT_H;
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			WZ = m_ea + 1;
			break;

		}
		break;

	case 0x44:
			neg();
		break;

	case 0x45:
		switch (u8(m_ref))
		{
		case 0x00:
			m_out_retn_seen_cb(0);
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			if (m_stackless) {
				m_pc.b.l = m_in_nextreg_cb(0xc2);
				m_pc.b.h = m_in_nextreg_cb(0xc3);
			} else {
				PC = TDAT;
			}
			LOGMASKED(LOG_INT, "RETN m_iff1:%d m_iff2:%d\n", m_iff1, m_iff2); WZ = PC; m_iff1 = m_iff2;
			break;

		}
		break;

	case 0x46:
			m_im = 0;
		break;

	case 0x47:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_i = A;
			break;

		}
		break;

	case 0x48:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			C = TDAT8; set_f((F & CF) | SZP[C]); WZ = TADR + 1;
			break;

		}
		break;

	case 0x49:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC; TDAT8 = C;
			[[fallthrough]];
		case 0x01:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = TADR + 1;
			break;

		}
		break;

	case 0x4a:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = BC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = HL + TDAT + (F & CF); WZ = HL + 1; set_f((((HL ^ res ^ TDAT) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (SF | YF | XF)) | ((res & 0xffff) ? 0 : ZF) | (((TDAT ^ HL ^ 0x8000) & (TDAT ^ res) & 0x8000) >> 13)); HL = (u16)res; }
			break;

		}
		break;

	case 0x4b:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea;
			[[fallthrough]];
		case 0x05:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TDAT_H = TDAT_L; TADR++;
			[[fallthrough]];
		case 0x07:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			using std::swap; std::swap(TDAT_H, TDAT_L);
			BC = TDAT; WZ = m_ea + 1;
			break;

		}
		break;

	case 0x4c:
			neg();
		break;

	case 0x4d:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			PC = TDAT; WZ = PC; m_iff1 = m_iff2;
			m_reti_cb(1);
			daisy_call_reti_device();
			break;

		}
		break;

	case 0x4e:
			m_im = 0;
		break;

	case 0x4f:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_r = A; m_r2 = A & 0x80; // keep bit 7 of r
			break;

		}
		break;

	case 0x50:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			D = TDAT8; set_f((F & CF) | SZP[D]); WZ = TADR + 1;
			break;

		}
		break;

	case 0x51:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC; TDAT8 = D;
			[[fallthrough]];
		case 0x01:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = TADR + 1;
			break;

		}
		break;

	case 0x52:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = DE;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = HL - TDAT - (F & CF); WZ = HL + 1; set_f((((HL ^ res ^ TDAT) >> 8) & HF) | NF | ((res >> 16) & CF) | ((res >> 8) & (SF | YF | XF)) | ((res & 0xffff) ? 0 : ZF) | (((TDAT ^ HL) & (HL ^ res) &0x8000) >> 13)); HL = (u16)res; }
			break;

		}
		break;

	case 0x53:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea; TDAT = DE;
			[[fallthrough]];
		case 0x05:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TADR++; TDAT8 = TDAT_H;
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			WZ = m_ea + 1;
			break;

		}
		break;

	case 0x54:
			neg();
		break;

	case 0x55:
		switch (u8(m_ref))
		{
		case 0x00:
			m_out_retn_seen_cb(0);
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			if (m_stackless) {
				m_pc.b.l = m_in_nextreg_cb(0xc2);
				m_pc.b.h = m_in_nextreg_cb(0xc3);
			} else {
				PC = TDAT;
			}
			LOGMASKED(LOG_INT, "RETN m_iff1:%d m_iff2:%d\n", m_iff1, m_iff2); WZ = PC; m_iff1 = m_iff2;
			break;

		}
		break;

	case 0x56:
			m_im = 1;
		break;

	case 0x57:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			A = m_i; set_f((F & CF) | SZ[A] | (m_iff2 << 2)); m_after_ldair = true;
			break;

		}
		break;

	case 0x58:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			E = TDAT8; set_f((F & CF) | SZP[E]); WZ = TADR + 1;
			break;

		}
		break;

	case 0x59:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC; TDAT8 = E;
			[[fallthrough]];
		case 0x01:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = TADR + 1;
			break;

		}
		break;

	case 0x5a:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = DE;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = HL + TDAT + (F & CF); WZ = HL + 1; set_f((((HL ^ res ^ TDAT) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (SF | YF | XF)) | ((res & 0xffff) ? 0 : ZF) | (((TDAT ^ HL ^ 0x8000) & (TDAT ^ res) & 0x8000) >> 13)); HL = (u16)res; }
			break;

		}
		break;

	case 0x5b:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea;
			[[fallthrough]];
		case 0x05:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TDAT_H = TDAT_L; TADR++;
			[[fallthrough]];
		case 0x07:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			using std::swap; std::swap(TDAT_H, TDAT_L);
			DE = TDAT; WZ = m_ea + 1;
			break;

		}
		break;

	case 0x5c:
			neg();
		break;

	case 0x5d:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			PC = TDAT; WZ = PC; m_iff1 = m_iff2;
			m_reti_cb(1);
			daisy_call_reti_device();
			break;

		}
		break;

	case 0x5e:
			m_im = 2;
		break;

	case 0x5f:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			A = (m_r & 0x7f) | m_r2; set_f((F & CF) | SZ[A] | (m_iff2 << 2)); m_after_ldair = true;
			break;

		}
		break;

	case 0x60:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			H = TDAT8; set_f((F & CF) | SZP[H]); WZ = TADR + 1;
			break;

		}
		break;

	case 0x61:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC; TDAT8 = H;
			[[fallthrough]];
		case 0x01:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = TADR + 1;
			break;

		}
		break;

	case 0x62:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = HL;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = HL - TDAT - (F & CF); WZ = HL + 1; set_f((((HL ^ res ^ TDAT) >> 8) & HF) | NF | ((res >> 16) & CF) | ((res >> 8) & (SF | YF | XF)) | ((res & 0xffff) ? 0 : ZF) | (((TDAT ^ HL) & (HL ^ res) &0x8000) >> 13)); HL = (u16)res; }
			break;

		}
		break;

	case 0x63:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea; TDAT = HL;
			[[fallthrough]];
		case 0x05:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TADR++; TDAT8 = TDAT_H;
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			WZ = m_ea + 1;
			break;

		}
		break;

	case 0x64:
			neg();
		break;

	case 0x65:
		switch (u8(m_ref))
		{
		case 0x00:
			m_out_retn_seen_cb(0);
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			if (m_stackless) {
				m_pc.b.l = m_in_nextreg_cb(0xc2);
				m_pc.b.h = m_in_nextreg_cb(0xc3);
			} else {
				PC = TDAT;
			}
			LOGMASKED(LOG_INT, "RETN m_iff1:%d m_iff2:%d\n", m_iff1, m_iff2); WZ = PC; m_iff1 = m_iff2;
			break;

		}
		break;

	case 0x66:
			m_im = 0;
		break;

	case 0x67:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = HL+1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TDAT_H = TDAT8; TDAT8 = (TDAT8 >> 4) | (A << 4);
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			A = (A & 0xf0) | (TDAT_H & 0x0f); set_f((F & CF) | SZP[A]);
			break;

		}
		break;

	case 0x68:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			L = TDAT8; set_f((F & CF) | SZP[L]); WZ = TADR + 1;
			break;

		}
		break;

	case 0x69:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC; TDAT8 = L;
			[[fallthrough]];
		case 0x01:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = TADR + 1;
			break;

		}
		break;

	case 0x6a:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = HL;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = HL + TDAT + (F & CF); WZ = HL + 1; set_f((((HL ^ res ^ TDAT) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (SF | YF | XF)) | ((res & 0xffff) ? 0 : ZF) | (((TDAT ^ HL ^ 0x8000) & (TDAT ^ res) & 0x8000) >> 13)); HL = (u16)res; }
			break;

		}
		break;

	case 0x6b:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea;
			[[fallthrough]];
		case 0x05:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TDAT_H = TDAT_L; TADR++;
			[[fallthrough]];
		case 0x07:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			using std::swap; std::swap(TDAT_H, TDAT_L);
			HL = TDAT; WZ = m_ea + 1;
			break;

		}
		break;

	case 0x6c:
			neg();
		break;

	case 0x6d:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			PC = TDAT; WZ = PC; m_iff1 = m_iff2;
			m_reti_cb(1);
			daisy_call_reti_device();
			break;

		}
		break;

	case 0x6e:
			m_im = 0;
		break;

	case 0x6f:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = HL+1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TDAT_H = TDAT8; TDAT8 = (TDAT8 << 4) | (A & 0x0f);
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			A = (A & 0xf0) | (TDAT_H >> 4); set_f((F & CF) | SZP[A]);
			break;

		}
		break;

	case 0x70:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			set_f((F & CF) | SZP[TDAT8]); WZ = TADR + 1;
			break;

		}
		break;

	case 0x71:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC; TDAT8 = 0;
			[[fallthrough]];
		case 0x01:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = TADR + 1;
			break;

		}
		break;

	case 0x72:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = SP;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = HL - TDAT - (F & CF); WZ = HL + 1; set_f((((HL ^ res ^ TDAT) >> 8) & HF) | NF | ((res >> 16) & CF) | ((res >> 8) & (SF | YF | XF)) | ((res & 0xffff) ? 0 : ZF) | (((TDAT ^ HL) & (HL ^ res) &0x8000) >> 13)); HL = (u16)res; }
			break;

		}
		break;

	case 0x73:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea; TDAT = SP;
			[[fallthrough]];
		case 0x05:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TADR++; TDAT8 = TDAT_H;
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			WZ = m_ea + 1;
			break;

		}
		break;

	case 0x74:
			neg();
		break;

	case 0x75:
		switch (u8(m_ref))
		{
		case 0x00:
			m_out_retn_seen_cb(0);
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			if (m_stackless) {
				m_pc.b.l = m_in_nextreg_cb(0xc2);
				m_pc.b.h = m_in_nextreg_cb(0xc3);
			} else {
				PC = TDAT;
			}
			LOGMASKED(LOG_INT, "RETN m_iff1:%d m_iff2:%d\n", m_iff1, m_iff2); WZ = PC; m_iff1 = m_iff2;
			break;

		}
		break;

	case 0x76:
			m_im = 1;
		break;

	case 0x77:
			illegal_2();
		break;

	case 0x78:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			A = TDAT8; set_f((F & CF) | SZP[A]); WZ = TADR + 1;
			break;

		}
		break;

	case 0x79:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC; TDAT8 = A;
			[[fallthrough]];
		case 0x01:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ = TADR + 1;
			break;

		}
		break;

	case 0x7a:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = SP;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = HL + TDAT + (F & CF); WZ = HL + 1; set_f((((HL ^ res ^ TDAT) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (SF | YF | XF)) | ((res & 0xffff) ? 0 : ZF) | (((TDAT ^ HL ^ 0x8000) & (TDAT ^ res) & 0x8000) >> 13)); HL = (u16)res; }
			break;

		}
		break;

	case 0x7b:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea;
			[[fallthrough]];
		case 0x05:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TDAT_H = TDAT_L; TADR++;
			[[fallthrough]];
		case 0x07:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			using std::swap; std::swap(TDAT_H, TDAT_L);
			SP = TDAT; WZ = m_ea + 1;
			break;

		}
		break;

	case 0x7c:
			neg();
		break;

	case 0x7d:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			PC = TDAT; WZ = PC; m_iff1 = m_iff2;
			m_reti_cb(1);
			daisy_call_reti_device();
			break;

		}
		break;

	case 0x7e:
			m_im = 2;
		break;

	case 0x7f:
			illegal_2();
		break;

	case 0x80:
			illegal_2();
		break;

	case 0x81:
			illegal_2();
		break;

	case 0x82:
			illegal_2();
		break;

	case 0x83:
			illegal_2();
		break;

	case 0x84:
			illegal_2();
		break;

	case 0x85:
			illegal_2();
		break;

	case 0x86:
			illegal_2();
		break;

	case 0x87:
			illegal_2();
		break;

	case 0x88:
			illegal_2();
		break;

	case 0x89:
			illegal_2();
		break;

	case 0x8a:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			using std::swap; swap(TDAT_H, TDAT_L);
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			break;

		}
		break;

	case 0x8b:
			illegal_2();
		break;

	case 0x8c:
			illegal_2();
		break;

	case 0x8d:
			illegal_2();
		break;

	case 0x8e:
			illegal_2();
		break;

	case 0x8f:
			illegal_2();
		break;

	case 0x90:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			TADR = BC;
			[[fallthrough]];
		case 0x03:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			HL++;
			break;

		}
		break;

	case 0x91:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_out_nextreg_cb(TDAT_L, TDAT_H);
			break;

		}
		break;

	case 0x92:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			m_out_nextreg_cb(TDAT8, A);
			break;

		}
		break;

	case 0x93:
		switch (u8(m_ref))
		{
		case 0x00:
			if (0x07 != (H & 0x07))
				HL = HL + 0x100;
			else if (0xe0 != (L & 0xe0))
				HL = (HL & 0xf8ff) + 0x20;
			else
				HL = (HL & 0xf81f) + 0x800;
			break;

		}
		break;

	case 0x94:
			HL = 0x4000 + ((D & 0xc0) << 5) + ((D & 0x07) << 8) + ((D & 0x38) << 2) + (E >> 3);
		break;

	case 0x95:
			A = 0x80 >> (E & 7);
		break;

	case 0x96:
			illegal_2();
		break;

	case 0x97:
			illegal_2();
		break;

	case 0x98:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC = (PC & 0xc000) + (TDAT8 << 6);
			break;

		}
		break;

	case 0x99:
			illegal_2();
		break;

	case 0x9a:
			illegal_2();
		break;

	case 0x9b:
			illegal_2();
		break;

	case 0x9c:
			illegal_2();
		break;

	case 0x9d:
			illegal_2();
		break;

	case 0x9e:
			illegal_2();
		break;

	case 0x9f:
			illegal_2();
		break;

	case 0xa0:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			set_f(F & (SF | ZF | CF)); if ((A + TDAT8) & 0x02) F |= YF;  if ((A + TDAT8) & 0x08) F |= XF; HL++; DE++; BC--; if(BC) F |= VF;
			break;

		}
		break;

	case 0xa1:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u8 res = A - TDAT8; WZ++; HL++; BC--; set_f((F & CF) | (SZ[res]&~(YF|XF)) | ((A^TDAT8^res)&HF) | NF); if (F & HF) res -= 1; if (res & 0x02) F |= YF; if (res & 0x08) F |= XF; if (BC) F |= VF; }
			break;

		}
		break;

	case 0xa2:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TADR = BC;
			[[fallthrough]];
		case 0x02:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			WZ = BC + 1; B--; TADR = HL;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			{ HL++; set_f(SZ[B]); unsigned t = (unsigned)((C + 1) & 0xff) + (unsigned)TDAT8; if (TDAT8 & SF) F |= NF; if (t & 0x100) F |= HF | CF; F |= SZP[(u8)(t & 0x07) ^ B] & PF; }
			break;

		}
		break;

	case 0xa3:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TADR = HL;
			[[fallthrough]];
		case 0x02:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B--; WZ = BC + 1; TADR = BC;
			[[fallthrough]];
		case 0x04:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			{ HL++; set_f(SZ[B]); unsigned t = (unsigned)L + (unsigned)TDAT8; if (TDAT8 & SF) F |= NF; if (t & 0x100) F |= HF | CF; F |= SZP[(u8)(t & 0x07) ^ B] & PF; }
			break;

		}
		break;

	case 0xa4:
		switch (u8(m_ref))
		{
		case 0x00:
			{
				TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
				if (TDAT8 != A) {
					TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
					;
				}
				HL++;
				DE++;
				BC--;
			}
			break;

		}
		break;

	case 0xa5:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			L++;
			inc(D);
			break;

		}
		break;

	case 0xa6:
			illegal_2();
		break;

	case 0xa7:
			illegal_2();
		break;

	case 0xa8:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			{ set_f(F & (SF | ZF | CF)); if ((A + TDAT8) & 0x02) F |= YF;  if ((A + TDAT8) & 0x08) F |= XF; HL--; DE--; BC--; if (BC) F |= VF; }
			break;

		}
		break;

	case 0xa9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			TADR = HL;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u8 res = A - TDAT8; WZ--; HL--; BC--; set_f((F & CF) | (SZ[res]&~(YF|XF)) | ((A^TDAT8^res)&HF) | NF); if (F & HF) res -= 1; if (res & 0x02) F |= YF; if (res & 0x08) F |= XF; if (BC) F |= VF; }
			break;

		}
		break;

	case 0xaa:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TADR = BC;
			[[fallthrough]];
		case 0x02:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			WZ = BC - 1; B--; TADR = HL;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			{ HL--; set_f(SZ[B]); unsigned t = ((unsigned)(C - 1) & 0xff) + (unsigned)TDAT8; if (TDAT8 & SF) F |= NF; if (t & 0x100) F |= HF | CF; F |= SZP[(u8)(t & 0x07) ^ B] & PF; }
			break;

		}
		break;

	case 0xab:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TADR = HL;
			[[fallthrough]];
		case 0x02:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B--; WZ = BC - 1; TADR = BC;
			[[fallthrough]];
		case 0x04:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			{ HL--; set_f(SZ[B]); unsigned t = (unsigned)L + (unsigned)TDAT8; if (TDAT8 & SF) F |= NF; if (t & 0x100) F |= HF | CF; F |= SZP[(u8)(t & 0x07) ^ B] & PF; }
			break;

		}
		break;

	case 0xac:
		switch (u8(m_ref))
		{
		case 0x00:
			{
				TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
				if (TDAT8 != A) {
					TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
					;
				}
				DE++;
				HL--;
				BC--;
			}
			break;

		}
		break;

	case 0xad:
			illegal_2();
		break;

	case 0xae:
			illegal_2();
		break;

	case 0xaf:
			illegal_2();
		break;

	case 0xb0:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			set_f(F & (SF | ZF | CF)); if ((A + TDAT8) & 0x02) F |= YF;  if ((A + TDAT8) & 0x08) F |= XF; HL++; DE++; BC--; if(BC) F |= VF;
			if (BC != 0) {
				TADR = DE;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
				PC -= 2; WZ = PC + 1; F &= ~(YF | XF); F |= (PC >> 8) & (YF | XF);
			}
			break;

		}
		break;

	case 0xb1:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u8 res = A - TDAT8; WZ++; HL++; BC--; set_f((F & CF) | (SZ[res]&~(YF|XF)) | ((A^TDAT8^res)&HF) | NF); if (F & HF) res -= 1; if (res & 0x02) F |= YF; if (res & 0x08) F |= XF; if (BC) F |= VF; }
			if (BC != 0 && !(F & ZF)) {
				TADR = HL;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0c;
				return;
			}
			[[fallthrough]];
		case 0x0c:
				PC -= 2; WZ = PC + 1; F &= ~(YF | XF); F |= (PC >> 8) & (YF | XF);
			}
			break;

		}
		break;

	case 0xb2:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TADR = BC;
			[[fallthrough]];
		case 0x02:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			WZ = BC + 1; B--; TADR = HL;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			{ HL++; set_f(SZ[B]); unsigned t = (unsigned)((C + 1) & 0xff) + (unsigned)TDAT8; if (TDAT8 & SF) F |= NF; if (t & 0x100) F |= HF | CF; F |= SZP[(u8)(t & 0x07) ^ B] & PF; }
			if (B != 0) {
				TADR = HL;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
				PC -= 2; block_io_interrupted_flags();
			}
			break;

		}
		break;

	case 0xb3:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TADR = HL;
			[[fallthrough]];
		case 0x02:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B--; WZ = BC + 1; TADR = BC;
			[[fallthrough]];
		case 0x04:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			{ HL++; set_f(SZ[B]); unsigned t = (unsigned)L + (unsigned)TDAT8; if (TDAT8 & SF) F |= NF; if (t & 0x100) F |= HF | CF; F |= SZP[(u8)(t & 0x07) ^ B] & PF; }
			if (B != 0) {
				TADR = BC;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
				PC -= 2; block_io_interrupted_flags();
			}
			break;

		}
		break;

	case 0xb4:
		switch (u8(m_ref))
		{
		case 0x00:
			{
				TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
				if (TDAT8 != A) {
					TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
					;
				}
				HL++;
				DE++;
				BC--;
			}
			if (BC != 0)
				PC -= 2;
			break;

		}
		break;

	case 0xb5:
			illegal_2();
		break;

	case 0xb6:
			illegal_2();
		break;

	case 0xb7:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (HL & 0xfff8) + (E & 7);
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			if (TDAT8 != A) {
				TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
				;
			}
			DE++;
			BC--;
			if (BC != 0)
				PC -= 2;
			break;

		}
		break;

	case 0xb8:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			{ set_f(F & (SF | ZF | CF)); if ((A + TDAT8) & 0x02) F |= YF;  if ((A + TDAT8) & 0x08) F |= XF; HL--; DE--; BC--; if (BC) F |= VF; }
			if (BC != 0) {
				TADR = DE;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
				PC -= 2; WZ = PC + 1; F &= ~(YF | XF); F |= (PC >> 8) & (YF | XF);
			}
			break;

		}
		break;

	case 0xb9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			TADR = HL;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u8 res = A - TDAT8; WZ--; HL--; BC--; set_f((F & CF) | (SZ[res]&~(YF|XF)) | ((A^TDAT8^res)&HF) | NF); if (F & HF) res -= 1; if (res & 0x02) F |= YF; if (res & 0x08) F |= XF; if (BC) F |= VF; }
			if (BC != 0 && !(F & ZF)) {
				TADR = HL;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0c;
				return;
			}
			[[fallthrough]];
		case 0x0c:
				PC -= 2; WZ = PC + 1; F &= ~(YF | XF); F |= (PC >> 8) & (YF | XF);
			}
			break;

		}
		break;

	case 0xba:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TADR = BC;
			[[fallthrough]];
		case 0x02:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			WZ = BC - 1; B--; TADR = HL;
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			{ HL--; set_f(SZ[B]); unsigned t = ((unsigned)(C - 1) & 0xff) + (unsigned)TDAT8; if (TDAT8 & SF) F |= NF; if (t & 0x100) F |= HF | CF; F |= SZP[(u8)(t & 0x07) ^ B] & PF; }
			if (B != 0) {
				TADR = HL;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
				PC -= 2; block_io_interrupted_flags();
			}
			break;

		}
		break;

	case 0xbb:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TADR = HL;
			[[fallthrough]];
		case 0x02:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			B--; WZ = BC - 1; TADR = BC;
			[[fallthrough]];
		case 0x04:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			{ HL--; set_f(SZ[B]); unsigned t = (unsigned)L + (unsigned)TDAT8; if (TDAT8 & SF) F |= NF; if (t & 0x100) F |= HF | CF; F |= SZP[(u8)(t & 0x07) ^ B] & PF; }
			if (B != 0) {
				TADR = BC;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
				PC -= 2; block_io_interrupted_flags();
			}
			break;

		}
		break;

	case 0xbc:
		switch (u8(m_ref))
		{
		case 0x00:
			{
				TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
				if (TDAT8 != A) {
					TADR = DE;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
					;
				}
				DE++;
				HL--;
				BC--;
			}
			if (BC != 0)
				PC -= 2;
			break;

		}
		break;

	case 0xbd:
			illegal_2();
		break;

	case 0xbe:
			illegal_2();
		break;

	case 0xbf:
			illegal_2();
		break;

	case 0xc0:
			illegal_2();
		break;

	case 0xc1:
			illegal_2();
		break;

	case 0xc2:
			illegal_2();
		break;

	case 0xc3:
			illegal_2();
		break;

	case 0xc4:
			illegal_2();
		break;

	case 0xc5:
			illegal_2();
		break;

	case 0xc6:
			illegal_2();
		break;

	case 0xc7:
			illegal_2();
		break;

	case 0xc8:
			illegal_2();
		break;

	case 0xc9:
			illegal_2();
		break;

	case 0xca:
			illegal_2();
		break;

	case 0xcb:
			illegal_2();
		break;

	case 0xcc:
			illegal_2();
		break;

	case 0xcd:
			illegal_2();
		break;

	case 0xce:
			illegal_2();
		break;

	case 0xcf:
			illegal_2();
		break;

	case 0xd0:
			illegal_2();
		break;

	case 0xd1:
			illegal_2();
		break;

	case 0xd2:
			illegal_2();
		break;

	case 0xd3:
			illegal_2();
		break;

	case 0xd4:
			illegal_2();
		break;

	case 0xd5:
			illegal_2();
		break;

	case 0xd6:
			illegal_2();
		break;

	case 0xd7:
			illegal_2();
		break;

	case 0xd8:
			illegal_2();
		break;

	case 0xd9:
			illegal_2();
		break;

	case 0xda:
			illegal_2();
		break;

	case 0xdb:
			illegal_2();
		break;

	case 0xdc:
			illegal_2();
		break;

	case 0xdd:
			illegal_2();
		break;

	case 0xde:
			illegal_2();
		break;

	case 0xdf:
			illegal_2();
		break;

	case 0xe0:
			illegal_2();
		break;

	case 0xe1:
			illegal_2();
		break;

	case 0xe2:
			illegal_2();
		break;

	case 0xe3:
			illegal_2();
		break;

	case 0xe4:
			illegal_2();
		break;

	case 0xe5:
			illegal_2();
		break;

	case 0xe6:
			illegal_2();
		break;

	case 0xe7:
			illegal_2();
		break;

	case 0xe8:
			illegal_2();
		break;

	case 0xe9:
			illegal_2();
		break;

	case 0xea:
			illegal_2();
		break;

	case 0xeb:
			illegal_2();
		break;

	case 0xec:
			illegal_2();
		break;

	case 0xed:
			illegal_2();
		break;

	case 0xee:
			illegal_2();
		break;

	case 0xef:
			illegal_2();
		break;

	case 0xf0:
			illegal_2();
		break;

	case 0xf1:
			illegal_2();
		break;

	case 0xf2:
			illegal_2();
		break;

	case 0xf3:
			illegal_2();
		break;

	case 0xf4:
			illegal_2();
		break;

	case 0xf5:
			illegal_2();
		break;

	case 0xf6:
			illegal_2();
		break;

	case 0xf7:
			illegal_2();
		break;

	case 0xf8:
			illegal_2();
		break;

	case 0xf9:
			illegal_2();
		break;

	case 0xfa:
			illegal_2();
		break;

	case 0xfb:
			illegal_2();
		break;

	case 0xfc:
			illegal_2();
		break;

	case 0xfd:
			illegal_2();
		break;

	case 0xfe:
			illegal_2();
		break;

	case 0xff:
			illegal_2();
		break;


	}
		break;

}
break; // prefix: 0xed

case 0x00:
{
	switch (u8(m_ref >> 8)) // opcode
	{
	case 0x00:
		break;

	case 0x01:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			BC = TDAT;
			break;

		}
		break;

	case 0x02:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC; TDAT8 = A;
			[[fallthrough]];
		case 0x01:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ_L = (BC + 1) & 0xFF;  WZ_H = A;
			break;

		}
		break;

	case 0x03:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			BC++;
			break;

		}
		break;

	case 0x04:
			inc(B);
		break;

	case 0x05:
			dec(B);
		break;

	case 0x06:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			B = TDAT8;
			break;

		}
		break;

	case 0x07:
			rlca();
		break;

	case 0x08:
			using std::swap; swap(m_af, m_af2);
		break;

	case 0x09:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = HL; TDAT2 = BC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			HL = TDAT;
			break;

		}
		break;

	case 0x0a:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = BC;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			A = TDAT8;  WZ = BC+1;
			break;

		}
		break;

	case 0x0b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			BC--;
			break;

		}
		break;

	case 0x0c:
			inc(C);
		break;

	case 0x0d:
			dec(C);
		break;

	case 0x0e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			C = TDAT8;
			break;

		}
		break;

	case 0x0f:
			rrca();
		break;

	case 0x10:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			TDAT8 = --B;
			if (TDAT8) {
			[[fallthrough]];
		case 0x02:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			PC += (s8)TDAT8; WZ = PC;
			m_branch_cb(1);
			} else {
			[[fallthrough]];
		case 0x09:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x09;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			PC++;
			}
			break;

		}
		break;

	case 0x11:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			DE = TDAT;
			break;

		}
		break;

	case 0x12:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = DE; TDAT8 = A;
			[[fallthrough]];
		case 0x01:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			WZ_L = (DE + 1) & 0xFF;  WZ_H = A;
			break;

		}
		break;

	case 0x13:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			DE++;
			break;

		}
		break;

	case 0x14:
			inc(D);
		break;

	case 0x15:
			dec(D);
		break;

	case 0x16:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			D = TDAT8;
			break;

		}
		break;

	case 0x17:
			rla();
		break;

	case 0x18:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			PC += (s8)TDAT8; WZ = PC;
			m_branch_cb(1);
			break;

		}
		break;

	case 0x19:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = HL; TDAT2 = DE;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			HL = TDAT;
			break;

		}
		break;

	case 0x1a:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = DE;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			A = TDAT8; WZ = DE + 1;
			break;

		}
		break;

	case 0x1b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			DE--;
			break;

		}
		break;

	case 0x1c:
			inc(E);
		break;

	case 0x1d:
			dec(E);
		break;

	case 0x1e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			E = TDAT8;
			break;

		}
		break;

	case 0x1f:
			rra();
		break;

	case 0x20:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & ZF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			PC += (s8)TDAT8; WZ = PC;
			m_branch_cb(1);
			} else {
			[[fallthrough]];
		case 0x08:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC++;
			}
			break;

		}
		break;

	case 0x21:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			HL = TDAT;
			break;

		}
		break;

	case 0x22:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = TDAT; TDAT = HL;
			[[fallthrough]];
		case 0x05:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TADR++; TDAT8 = TDAT_H;
			[[fallthrough]];
		case 0x07:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			WZ = m_ea + 1;
			break;

		}
		break;

	case 0x23:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			HL++;
			break;

		}
		break;

	case 0x24:
			inc(H);
		break;

	case 0x25:
			dec(H);
		break;

	case 0x26:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			H = TDAT8;
			break;

		}
		break;

	case 0x27:
			daa();
		break;

	case 0x28:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & ZF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			PC += (s8)TDAT8; WZ = PC;
			m_branch_cb(1);
			} else {
			[[fallthrough]];
		case 0x08:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC++;
			}
			break;

		}
		break;

	case 0x29:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = HL; TDAT2 = HL;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			HL = TDAT;
			break;

		}
		break;

	case 0x2a:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = TDAT;
			[[fallthrough]];
		case 0x05:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			TDAT_H = TDAT_L; TADR++;
			[[fallthrough]];
		case 0x07:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			using std::swap; std::swap(TDAT_H, TDAT_L);
			HL = TDAT; WZ = m_ea + 1;
			break;

		}
		break;

	case 0x2b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			HL--;
			break;

		}
		break;

	case 0x2c:
			inc(L);
		break;

	case 0x2d:
			dec(L);
		break;

	case 0x2e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			L = TDAT8;
			break;

		}
		break;

	case 0x2f:
			A ^= 0xff; set_f((F & (SF | ZF | PF | CF)) | HF | NF | (A & (YF | XF)));
		break;

	case 0x30:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & CF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			PC += (s8)TDAT8; WZ = PC;
			m_branch_cb(1);
			} else {
			[[fallthrough]];
		case 0x08:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC++;
			}
			break;

		}
		break;

	case 0x31:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			SP = TDAT;
			break;

		}
		break;

	case 0x32:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = m_ea; TDAT8 = A;
			[[fallthrough]];
		case 0x05:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			WZ_L = (m_ea + 1) & 0xFF; WZ_H = A;
			break;

		}
		break;

	case 0x33:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			break;

		}
		break;

	case 0x34:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			inc(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x35:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			dec(TDAT8);
			[[fallthrough]];
		case 0x04:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0x36:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TADR = HL;
			[[fallthrough]];
		case 0x03:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			break;

		}
		break;

	case 0x37:
			set_f((F & (SF | ZF | PF)) | CF | (((F & Q) | A) & (YF | XF)));
		break;

	case 0x38:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & CF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			PC += (s8)TDAT8; WZ = PC;
			m_branch_cb(1);
			} else {
			[[fallthrough]];
		case 0x08:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC++;
			}
			break;

		}
		break;

	case 0x39:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = HL; TDAT2 = SP;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			{ u32 res = TDAT + TDAT2; WZ = TDAT + 1; set_f((F & (SF | ZF | VF)) | (((TDAT ^ res ^ TDAT2) >> 8) & HF) | ((res >> 16) & CF) | ((res >> 8) & (YF | XF))); TDAT = (u16)res; }
			HL = TDAT;
			break;

		}
		break;

	case 0x3a:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = TDAT;
			[[fallthrough]];
		case 0x05:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			A = TDAT8; WZ = m_ea + 1;
			break;

		}
		break;

	case 0x3b:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP--;
			break;

		}
		break;

	case 0x3c:
			inc(A);
		break;

	case 0x3d:
			dec(A);
		break;

	case 0x3e:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			A = TDAT8;
			break;

		}
		break;

	case 0x3f:
			set_f(((F & (SF | ZF | PF | CF)) ^ CF) | ((F & CF) << 4) | (((F & Q) | A) & (YF | XF)));
		break;

	case 0x40:
		break;

	case 0x41:
			B = C;
		break;

	case 0x42:
			B = D;
		break;

	case 0x43:
			B = E;
		break;

	case 0x44:
			B = H;
		break;

	case 0x45:
			B = L;
		break;

	case 0x46:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			B = TDAT8;
			break;

		}
		break;

	case 0x47:
			B = A;
		break;

	case 0x48:
			C = B;
		break;

	case 0x49:
		break;

	case 0x4a:
			C = D;
		break;

	case 0x4b:
			C = E;
		break;

	case 0x4c:
			C = H;
		break;

	case 0x4d:
			C = L;
		break;

	case 0x4e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			C = TDAT8;
			break;

		}
		break;

	case 0x4f:
			C = A;
		break;

	case 0x50:
			D = B;
		break;

	case 0x51:
			D = C;
		break;

	case 0x52:
		break;

	case 0x53:
			D = E;
		break;

	case 0x54:
			D = H;
		break;

	case 0x55:
			D = L;
		break;

	case 0x56:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			D = TDAT8;
			break;

		}
		break;

	case 0x57:
			D = A;
		break;

	case 0x58:
			E = B;
		break;

	case 0x59:
			E = C;
		break;

	case 0x5a:
			E = D;
		break;

	case 0x5b:
		break;

	case 0x5c:
			E = H;
		break;

	case 0x5d:
			E = L;
		break;

	case 0x5e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			E = TDAT8;
			break;

		}
		break;

	case 0x5f:
			E = A;
		break;

	case 0x60:
			H = B;
		break;

	case 0x61:
			H = C;
		break;

	case 0x62:
			H = D;
		break;

	case 0x63:
			H = E;
		break;

	case 0x64:
		break;

	case 0x65:
			H = L;
		break;

	case 0x66:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			H = TDAT8;
			break;

		}
		break;

	case 0x67:
			H = A;
		break;

	case 0x68:
			L = B;
		break;

	case 0x69:
			L = C;
		break;

	case 0x6a:
			L = D;
		break;

	case 0x6b:
			L = E;
		break;

	case 0x6c:
			L = H;
		break;

	case 0x6d:
		break;

	case 0x6e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			L = TDAT8;
			break;

		}
		break;

	case 0x6f:
			L = A;
		break;

	case 0x70:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL; TDAT = B;
			[[fallthrough]];
		case 0x01:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			break;

		}
		break;

	case 0x71:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL; TDAT = C;
			[[fallthrough]];
		case 0x01:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			break;

		}
		break;

	case 0x72:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL; TDAT = D;
			[[fallthrough]];
		case 0x01:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			break;

		}
		break;

	case 0x73:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL; TDAT = E;
			[[fallthrough]];
		case 0x01:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			break;

		}
		break;

	case 0x74:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL; TDAT = H;
			[[fallthrough]];
		case 0x01:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			break;

		}
		break;

	case 0x75:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL; TDAT = L;
			[[fallthrough]];
		case 0x01:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			break;

		}
		break;

	case 0x76:
			halt();
		break;

	case 0x77:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL; TDAT = A;
			[[fallthrough]];
		case 0x01:
			data_write(TADR, TDAT8);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			break;

		}
		break;

	case 0x78:
			A = B;
		break;

	case 0x79:
			A = C;
		break;

	case 0x7a:
			A = D;
		break;

	case 0x7b:
			A = E;
		break;

	case 0x7c:
			A = H;
		break;

	case 0x7d:
			A = L;
		break;

	case 0x7e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			A = TDAT8;
			break;

		}
		break;

	case 0x7f:
		break;

	case 0x80:
			add_a(B);
		break;

	case 0x81:
			add_a(C);
		break;

	case 0x82:
			add_a(D);
		break;

	case 0x83:
			add_a(E);
		break;

	case 0x84:
			add_a(H);
		break;

	case 0x85:
			add_a(L);
		break;

	case 0x86:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			add_a(TDAT8);
			break;

		}
		break;

	case 0x87:
			add_a(A);
		break;

	case 0x88:
			adc_a(B);
		break;

	case 0x89:
			adc_a(C);
		break;

	case 0x8a:
			adc_a(D);
		break;

	case 0x8b:
			adc_a(E);
		break;

	case 0x8c:
			adc_a(H);
		break;

	case 0x8d:
			adc_a(L);
		break;

	case 0x8e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			adc_a(TDAT8);
			break;

		}
		break;

	case 0x8f:
			adc_a(A);
		break;

	case 0x90:
			sub(B);
		break;

	case 0x91:
			sub(C);
		break;

	case 0x92:
			sub(D);
		break;

	case 0x93:
			sub(E);
		break;

	case 0x94:
			sub(H);
		break;

	case 0x95:
			sub(L);
		break;

	case 0x96:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			sub(TDAT8);
			break;

		}
		break;

	case 0x97:
			sub(A);
		break;

	case 0x98:
			sbc_a(B);
		break;

	case 0x99:
			sbc_a(C);
		break;

	case 0x9a:
			sbc_a(D);
		break;

	case 0x9b:
			sbc_a(E);
		break;

	case 0x9c:
			sbc_a(H);
		break;

	case 0x9d:
			sbc_a(L);
		break;

	case 0x9e:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			sbc_a(TDAT8);
			break;

		}
		break;

	case 0x9f:
			sbc_a(A);
		break;

	case 0xa0:
			and_a(B);
		break;

	case 0xa1:
			and_a(C);
		break;

	case 0xa2:
			and_a(D);
		break;

	case 0xa3:
			and_a(E);
		break;

	case 0xa4:
			and_a(H);
		break;

	case 0xa5:
			and_a(L);
		break;

	case 0xa6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			and_a(TDAT8);
			break;

		}
		break;

	case 0xa7:
			and_a(A);
		break;

	case 0xa8:
			xor_a(B);
		break;

	case 0xa9:
			xor_a(C);
		break;

	case 0xaa:
			xor_a(D);
		break;

	case 0xab:
			xor_a(E);
		break;

	case 0xac:
			xor_a(H);
		break;

	case 0xad:
			xor_a(L);
		break;

	case 0xae:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			xor_a(TDAT8);
			break;

		}
		break;

	case 0xaf:
			xor_a(A);
		break;

	case 0xb0:
			or_a(B);
		break;

	case 0xb1:
			or_a(C);
		break;

	case 0xb2:
			or_a(D);
		break;

	case 0xb3:
			or_a(E);
		break;

	case 0xb4:
			or_a(H);
		break;

	case 0xb5:
			or_a(L);
		break;

	case 0xb6:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			or_a(TDAT8);
			break;

		}
		break;

	case 0xb7:
			or_a(A);
		break;

	case 0xb8:
			cp(B);
		break;

	case 0xb9:
			cp(C);
		break;

	case 0xba:
			cp(D);
		break;

	case 0xbb:
			cp(E);
		break;

	case 0xbc:
			cp(H);
		break;

	case 0xbd:
			cp(L);
		break;

	case 0xbe:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = HL;
			[[fallthrough]];
		case 0x01:
			TDAT8 = data_read(TADR);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			cp(TDAT8);
			break;

		}
		break;

	case 0xbf:
			cp(A);
		break;

	case 0xc0:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & ZF);
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			if (TDAT8) {
			[[fallthrough]];
		case 0x02:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP++;
			[[fallthrough]];
		case 0x04:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			SP++;
				PC = TDAT; WZ = PC;
			}
			break;

		}
		break;

	case 0xc1:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			BC = TDAT;
			break;

		}
		break;

	case 0xc2:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & ZF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				PC = TDAT; WZ = PC;
				m_branch_cb(1);
			} else {
				// implicit do PC += 2
			[[fallthrough]];
		case 0x05:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x07:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xc3:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			PC = TDAT; WZ = PC;
			m_branch_cb(1);
			break;

		}
		break;

	case 0xc4:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & ZF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			WZ = m_ea; TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC = m_ea;
			} else {
			[[fallthrough]];
		case 0x0a:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0a;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x0c:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0d;
				return;
			}
			[[fallthrough]];
		case 0x0d:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xc5:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = BC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xc6:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			add_a(TDAT8);
			break;

		}
		break;

	case 0xc7:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = PC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			PC = 0x00; WZ = PC;
			break;

		}
		break;

	case 0xc8:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = (F & ZF);
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			if (TDAT8) {
			[[fallthrough]];
		case 0x02:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP++;
			[[fallthrough]];
		case 0x04:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			SP++;
				PC = TDAT; WZ = PC;
			}
			break;

		}
		break;

	case 0xc9:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			PC = TDAT; WZ = PC;
			break;

		}
		break;

	case 0xca:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & ZF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				PC = TDAT; WZ = PC;
				m_branch_cb(1);
			} else {
				// implicit do PC += 2
			[[fallthrough]];
		case 0x05:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x07:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xcb:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = opcode_read();
			m_icount -= m_m1_cycles-2;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_m1_cycles-2;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_refresh_cb((I << 8) | (R2 & 0x80) | (R & 0x7f), 0x00, 0xff);
			m_icount -= 2;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			PC++; R++; Q = QT; QT = YF | XF;
			m_ref = (0xcb << 16) | (TDAT8 << 8);
			if (true) return;
			break;

		}
		break;

	case 0xcc:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & ZF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			WZ = m_ea; TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC = m_ea;
			} else {
			[[fallthrough]];
		case 0x0a:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0a;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x0c:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0d;
				return;
			}
			[[fallthrough]];
		case 0x0d:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xcd:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			WZ = m_ea; TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC = m_ea;
			break;

		}
		break;

	case 0xce:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			adc_a(TDAT8);
			break;

		}
		break;

	case 0xcf:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = PC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			PC = 0x08; WZ = PC;
			break;

		}
		break;

	case 0xd0:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & CF);
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			if (TDAT8) {
			[[fallthrough]];
		case 0x02:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP++;
			[[fallthrough]];
		case 0x04:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			SP++;
				PC = TDAT; WZ = PC;
			}
			break;

		}
		break;

	case 0xd1:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			DE = TDAT;
			break;

		}
		break;

	case 0xd2:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & CF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				PC = TDAT; WZ = PC;
				m_branch_cb(1);
			} else {
				// implicit do PC += 2
			[[fallthrough]];
		case 0x05:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x07:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xd3:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TADR = TDAT8 | (A << 8); TDAT = A;
			[[fallthrough]];
		case 0x03:
			m_io.write_interruptible(TADR, TDAT8);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			WZ_L = ((TADR & 0xff) + 1) & 0xff;  WZ_H = A;
			break;

		}
		break;

	case 0xd4:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & CF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			WZ = m_ea; TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC = m_ea;
			} else {
			[[fallthrough]];
		case 0x0a:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0a;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x0c:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0d;
				return;
			}
			[[fallthrough]];
		case 0x0d:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xd5:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = DE;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xd6:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			sub(TDAT8);
			break;

		}
		break;

	case 0xd7:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = PC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			PC = 0x10; WZ = PC;
			break;

		}
		break;

	case 0xd8:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = (F & CF);
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			if (TDAT8) {
			[[fallthrough]];
		case 0x02:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP++;
			[[fallthrough]];
		case 0x04:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			SP++;
				PC = TDAT; WZ = PC;
			}
			break;

		}
		break;

	case 0xd9:
			exx();
		break;

	case 0xda:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & CF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				PC = TDAT; WZ = PC;
				m_branch_cb(1);
			} else {
				// implicit do PC += 2
			[[fallthrough]];
		case 0x05:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x07:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xdb:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TADR = TDAT8 | (A << 8);
			[[fallthrough]];
		case 0x03:
			TDAT8 = m_io.read_interruptible(TADR);
			m_icount -= m_iorq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_iorq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			A = TDAT8; WZ = TADR + 1;
			break;

		}
		break;

	case 0xdc:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & CF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			WZ = m_ea; TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC = m_ea;
			} else {
			[[fallthrough]];
		case 0x0a:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0a;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x0c:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0d;
				return;
			}
			[[fallthrough]];
		case 0x0d:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xdd:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = opcode_read();
			m_icount -= m_m1_cycles-2;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_m1_cycles-2;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_refresh_cb((I << 8) | (R2 & 0x80) | (R & 0x7f), 0x00, 0xff);
			m_icount -= 2;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			PC++; R++; Q = QT; QT = YF | XF;
			m_ref = (0xdd << 16) | (TDAT8 << 8);
			if (true) return;
			break;

		}
		break;

	case 0xde:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			sbc_a(TDAT8);
			break;

		}
		break;

	case 0xdf:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = PC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			PC = 0x18; WZ = PC;
			break;

		}
		break;

	case 0xe0:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & PF);
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			if (TDAT8) {
			[[fallthrough]];
		case 0x02:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP++;
			[[fallthrough]];
		case 0x04:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			SP++;
				PC = TDAT; WZ = PC;
			}
			break;

		}
		break;

	case 0xe1:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			HL = TDAT;
			break;

		}
		break;

	case 0xe2:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & PF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				PC = TDAT; WZ = PC;
				m_branch_cb(1);
			} else {
				// implicit do PC += 2
			[[fallthrough]];
		case 0x05:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x07:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xe3:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = HL;
			TDAT2 = TDAT;
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			TADR = SP-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			using std::swap; std::swap(TDAT, TDAT2);
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			TADR = SP;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0a;
				return;
			}
			[[fallthrough]];
		case 0x0a:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			using std::swap; std::swap(TDAT, TDAT2); WZ = TDAT;
			HL = TDAT;
			break;

		}
		break;

	case 0xe4:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & PF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			WZ = m_ea; TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC = m_ea;
			} else {
			[[fallthrough]];
		case 0x0a:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0a;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x0c:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0d;
				return;
			}
			[[fallthrough]];
		case 0x0d:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xe5:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = HL;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xe6:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			and_a(TDAT8);
			break;

		}
		break;

	case 0xe7:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = PC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			PC = 0x20; WZ = PC;
			break;

		}
		break;

	case 0xe8:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = (F & PF);
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			if (TDAT8) {
			[[fallthrough]];
		case 0x02:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP++;
			[[fallthrough]];
		case 0x04:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			SP++;
				PC = TDAT; WZ = PC;
			}
			break;

		}
		break;

	case 0xe9:
			PC = HL;
		break;

	case 0xea:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & PF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				PC = TDAT; WZ = PC;
				m_branch_cb(1);
			} else {
				// implicit do PC += 2
			[[fallthrough]];
		case 0x05:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x07:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xeb:
			using std::swap; swap(DE, HL);
		break;

	case 0xec:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & PF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			WZ = m_ea; TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC = m_ea;
			} else {
			[[fallthrough]];
		case 0x0a:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0a;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x0c:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0d;
				return;
			}
			[[fallthrough]];
		case 0x0d:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xed:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = opcode_read();
			m_icount -= m_m1_cycles-2;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_m1_cycles-2;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_refresh_cb((I << 8) | (R2 & 0x80) | (R & 0x7f), 0x00, 0xff);
			m_icount -= 2;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			PC++; R++; Q = QT; QT = YF | XF;
			m_ref = (0xed << 16) | (TDAT8 << 8);
			if (true) return;
			break;

		}
		break;

	case 0xee:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			xor_a(TDAT8);
			break;

		}
		break;

	case 0xef:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = PC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			PC = 0x28; WZ = PC;
			break;

		}
		break;

	case 0xf0:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & SF);
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			if (TDAT8) {
			[[fallthrough]];
		case 0x02:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP++;
			[[fallthrough]];
		case 0x04:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			SP++;
				PC = TDAT; WZ = PC;
			}
			break;

		}
		break;

	case 0xf1:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP++;
			[[fallthrough]];
		case 0x03:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			SP++;
			AF = TDAT;
			break;

		}
		break;

	case 0xf2:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & SF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				PC = TDAT; WZ = PC;
				m_branch_cb(1);
			} else {
				// implicit do PC += 2
			[[fallthrough]];
		case 0x05:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x07:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xf3:
			m_iff1 = m_iff2 = 0;
		break;

	case 0xf4:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = !(F & SF);
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			WZ = m_ea; TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC = m_ea;
			} else {
			[[fallthrough]];
		case 0x0a:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0a;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x0c:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0d;
				return;
			}
			[[fallthrough]];
		case 0x0d:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xf5:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = AF;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			break;

		}
		break;

	case 0xf6:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			or_a(TDAT8);
			break;

		}
		break;

	case 0xf7:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = PC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			PC = 0x30; WZ = PC;
			break;

		}
		break;

	case 0xf8:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = (F & SF);
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			if (TDAT8) {
			[[fallthrough]];
		case 0x02:
			TDAT_L = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP++;
			[[fallthrough]];
		case 0x04:
			TDAT_H = stack_read(SP);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			SP++;
				PC = TDAT; WZ = PC;
			}
			break;

		}
		break;

	case 0xf9:
		switch (u8(m_ref))
		{
		case 0x00:
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			SP = HL;
			break;

		}
		break;

	case 0xfa:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & SF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				PC = TDAT; WZ = PC;
				m_branch_cb(1);
			} else {
				// implicit do PC += 2
			[[fallthrough]];
		case 0x05:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x05;
				} else
					m_ref = (m_ref & 0xffff00) | 0x06;
				return;
			}
			[[fallthrough]];
		case 0x06:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x07:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x07;
				} else
					m_ref = (m_ref & 0xffff00) | 0x08;
				return;
			}
			[[fallthrough]];
		case 0x08:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xfb:
			ei();
		break;

	case 0xfc:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT8 = F & SF;
			if (TDAT8) {
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x03:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x03;
				} else
					m_ref = (m_ref & 0xffff00) | 0x04;
				return;
			}
			[[fallthrough]];
		case 0x04:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
			m_ea = TDAT; TADR = PC-1;
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			WZ = m_ea; TDAT = PC;
			SP--;
			[[fallthrough]];
		case 0x06:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x06;
				} else
					m_ref = (m_ref & 0xffff00) | 0x07;
				return;
			}
			[[fallthrough]];
		case 0x07:
			SP--;
			[[fallthrough]];
		case 0x08:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x08;
				} else
					m_ref = (m_ref & 0xffff00) | 0x09;
				return;
			}
			[[fallthrough]];
		case 0x09:
			PC = m_ea;
			} else {
			[[fallthrough]];
		case 0x0a:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0a;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0b;
				return;
			}
			[[fallthrough]];
		case 0x0b:
			PC++;
			TDAT_H = TDAT_L;
			[[fallthrough]];
		case 0x0c:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x0c;
				} else
					m_ref = (m_ref & 0xffff00) | 0x0d;
				return;
			}
			[[fallthrough]];
		case 0x0d:
			PC++;
			using std::swap; swap(TDAT_H, TDAT_L);
				WZ = TDAT;
			}
			break;

		}
		break;

	case 0xfd:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = opcode_read();
			m_icount -= m_m1_cycles-2;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_m1_cycles-2;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			m_refresh_cb((I << 8) | (R2 & 0x80) | (R & 0x7f), 0x00, 0xff);
			m_icount -= 2;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			PC++; R++; Q = QT; QT = YF | XF;
			m_ref = (0xfd << 16) | (TDAT8 << 8);
			if (true) return;
			break;

		}
		break;

	case 0xfe:
		switch (u8(m_ref))
		{
		case 0x00:
			[[fallthrough]];
		case 0x01:
			TDAT8 = arg_read();
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x01;
				} else
					m_ref = (m_ref & 0xffff00) | 0x02;
				return;
			}
			[[fallthrough]];
		case 0x02:
			PC++;
			cp(TDAT8);
			break;

		}
		break;

	case 0xff:
		switch (u8(m_ref))
		{
		case 0x00:
			TDAT = PC;
			TADR = (m_i << 8) | (m_r2 & 0x80) | (m_r & 0x7f);
			m_nomreq_cb(TADR, 0x00, 0xff);
			m_icount -= 1;
			if (m_icount <= 0) {
				m_ref = (m_ref & 0xffff00) | 0x01;
				return;
			}
			[[fallthrough]];
		case 0x01:
			SP--;
			[[fallthrough]];
		case 0x02:
			stack_write(SP, TDAT_H);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x02;
				} else
					m_ref = (m_ref & 0xffff00) | 0x03;
				return;
			}
			[[fallthrough]];
		case 0x03:
			SP--;
			[[fallthrough]];
		case 0x04:
			stack_write(SP, TDAT_L);
			m_icount -= m_memrq_cycles;
			if (m_icount <= 0) {
				if (access_to_be_redone()) {
					m_icount += m_memrq_cycles;
					m_ref = (m_ref & 0xffff00) | 0x04;
				} else
					m_ref = (m_ref & 0xffff00) | 0x05;
				return;
			}
			[[fallthrough]];
		case 0x05:
			PC = 0x38; WZ = PC;
			break;

		}
		break;

	} // switch opcode
}
break; // prefix: 0x00

} // switch prefix

m_ref = 0xffff00;
